searchData={"items":[{"type":"module","title":"erlfdb","doc":"The primary API for interacting with a FoundationDB database.\n\nPlease refer to the [FoundationDB C API documentation](https://apple.github.io/foundationdb/api-c.html)\nfor the full specification of the functions we use. In this module documentation,\nwe won't duplicate all the content from the C API docs. Instead, we'll point out any differences\nwhere relevant, and provide usage examples.\n\nYou will usually be interacting with erlfdb functions in the proximity of an\n[FDB Transaction](https://apple.github.io/foundationdb/developer-guide.html#transaction-basics).\nThis example shows a basic transaction. See `transactional/2` for more.\n\n<!-- tabs-open -->","ref":"erlfdb.html"},{"type":"module","title":"Erlang - erlfdb","doc":"```erlang\n% Writes to <<\"foo\">> if it doesn't exist, and keeps track of the number of times we do so.\n1> Db = erlfdb:open().\n2> erlfdb:transactional(Db, fun(Tx) ->\n..   Future = erlfdb:get(Tx, <<\"foo\">>),\n..   case erlfdb:wait(Future) of\n..       not_found ->\n..           erlfdb:add(Tx, <<\"count\">>, 1),\n..           erlfdb:set(Tx, <<\"foo\">>, <<\"bar\">>);\n..       _Val ->\n..           ok\n..   end\n.. end).\n```","ref":"erlfdb.html#module-erlang"},{"type":"module","title":"Elixir - erlfdb","doc":"```elixir\n# Writes to \"foo\" if it doesn't exist, and keeps track of the number of times we do so.\niex> db = :erlfdb.open()\niex> :erlfdb.transactional(db, fn tx ->\n...>\n...>   val = tx\n...>         |> :erlfdb.get(\"foo\")\n...>         |> :erlfdb.wait()\n...>\n...>   case val do\n...>       :not_found ->\n...>           :erlfdb.add(tx, \"count\", 1)\n...>           :erlfdb.set(tx, \"foo\", \"bar\")\n...>       _val ->\n...>           :ok\n...>   end\n...> end).\n```\n\n<!-- tabs-close -->","ref":"erlfdb.html#module-elixir"},{"type":"function","title":"erlfdb.add/3","doc":"Performs an atomic add on the value in the database.\n\n*C API function*: [`fdb_transaction_atomic_op`](https://apple.github.io/foundationdb/api-c.html#c.fdb_transaction_atomic_op)\n\nSee `FDBMutationType.FDB_MUTATION_TYPE_ADD`.","ref":"erlfdb.html#add/3"},{"type":"function","title":"erlfdb.add_conflict_range/4","doc":"Adds a conflict to the transaction without actually performing the associated action.\n\nThis is not needed in simple cases.\n\n*C API function*: [`fdb_transaction_add_conflict_range`](https://apple.github.io/foundationdb/api-c.html#c.fdb_transaction_add_conflict_range)","ref":"erlfdb.html#add_conflict_range/4"},{"type":"function","title":"erlfdb.add_read_conflict_key/2","doc":"Adds a read conflict to the transaction without actually reading the key.\n\nThis is not needed in simple cases.\n\n*C API function*: [`fdb_transaction_add_conflict_range`](https://apple.github.io/foundationdb/api-c.html#c.fdb_transaction_add_conflict_range)","ref":"erlfdb.html#add_read_conflict_key/2"},{"type":"function","title":"erlfdb.add_read_conflict_range/3","doc":"Adds a read conflict to the transaction without actually reading the range.\n\nThis is not needed in simple cases.\n\n*C API function*: [`fdb_transaction_add_conflict_range`](https://apple.github.io/foundationdb/api-c.html#c.fdb_transaction_add_conflict_range)","ref":"erlfdb.html#add_read_conflict_range/3"},{"type":"function","title":"erlfdb.add_write_conflict_key/2","doc":"Adds a write conflict to the transaction without actually writing to the key.\n\nThis is not needed in simple cases.\n\n*C API function*: [`fdb_transaction_add_conflict_range`](https://apple.github.io/foundationdb/api-c.html#c.fdb_transaction_add_conflict_range)","ref":"erlfdb.html#add_write_conflict_key/2"},{"type":"function","title":"erlfdb.add_write_conflict_range/3","doc":"Adds a write conflict to the transaction without actually writing to the range.\n\nThis is not needed in simple cases.\n\n*C API function*: [`fdb_transaction_add_conflict_range`](https://apple.github.io/foundationdb/api-c.html#c.fdb_transaction_add_conflict_range)","ref":"erlfdb.html#add_write_conflict_range/3"},{"type":"function","title":"erlfdb.atomic_op/4","doc":"Performs one of the available atomic operations.\n\n*C API function*: [`fdb_transaction_atomic_op`](https://apple.github.io/foundationdb/api-c.html#c.fdb_transaction_atomic_op)","ref":"erlfdb.html#atomic_op/4"},{"type":"function","title":"erlfdb.bit_and/3","doc":"Performs an atomic bitwise 'and' on the value in the database.\n\n*C API function*: [`fdb_transaction_atomic_op`](https://apple.github.io/foundationdb/api-c.html#c.fdb_transaction_atomic_op)\n\nSee `FDBMutationType.FDB_MUTATION_TYPE_AND`.","ref":"erlfdb.html#bit_and/3"},{"type":"function","title":"erlfdb.bit_or/3","doc":"Performs an atomic bitwise 'or' on the value in the database.\n\n*C API function*: [`fdb_transaction_atomic_op`](https://apple.github.io/foundationdb/api-c.html#c.fdb_transaction_atomic_op)\n\nSee `FDBMutationType.FDB_MUTATION_TYPE_OR`.","ref":"erlfdb.html#bit_or/3"},{"type":"function","title":"erlfdb.bit_xor/3","doc":"Performs an atomic bitwise 'xor' on the value in the database.\n\n*C API function*: [`fdb_transaction_atomic_op`](https://apple.github.io/foundationdb/api-c.html#c.fdb_transaction_atomic_op)\n\nSee `FDBMutationType.FDB_MUTATION_TYPE_XOR`.","ref":"erlfdb.html#bit_xor/3"},{"type":"function","title":"erlfdb.block_until_ready/1","doc":"Blocks the calling process until the future is ready.\n\n*C API function*: [`fdb_future_block_until_ready`](https://apple.github.io/foundationdb/api-c.html#c.fdb_future_block_until_ready)\n\n> #### Tip {: .tip}\n>\n> Use one of the `wait*` functions instead, such as `wait/1`.","ref":"erlfdb.html#block_until_ready/1"},{"type":"function","title":"erlfdb.byte_max/3","doc":"Performs an atomic lexocogrpahic maximum on the value in the database.\n\n*C API function*: [`fdb_transaction_atomic_op`](https://apple.github.io/foundationdb/api-c.html#c.fdb_transaction_atomic_op)\n\nSee `FDBMutationType.FDB_MUTATION_TYPE_BYTE_MAX`.","ref":"erlfdb.html#byte_max/3"},{"type":"function","title":"erlfdb.byte_min/3","doc":"Performs an atomic lexocogrpahic minimum on the value in the database.\n\n*C API function*: [`fdb_transaction_atomic_op`](https://apple.github.io/foundationdb/api-c.html#c.fdb_transaction_atomic_op)\n\nSee `FDBMutationType.FDB_MUTATION_TYPE_BYTE_MIN`.","ref":"erlfdb.html#byte_min/3"},{"type":"function","title":"erlfdb.cancel/1","doc":"Cancels a future or a transaction.\n\n*C API functions*:\n\n- For a `t:transaction/0`, [`fdb_transaction_cancel`](https://apple.github.io/foundationdb/api-c.html#c.fdb_transaction_cancel)\n- For a `t:future/0` or `t:fold_future/0`, [`fdb_future_cancel`](https://apple.github.io/foundationdb/api-c.html#c.fdb_future_cancel)","ref":"erlfdb.html#cancel/1"},{"type":"function","title":"erlfdb.cancel/2","doc":"Cancels a future.\n\nIf `[{flush, true}]` is provided, the calling process's\nmailbox is cleared of any messages from this future. Defaults to `false`.\n\n*C API functions*:\n\n- For a `t:transaction/0`, [`fdb_transaction_cancel`](https://apple.github.io/foundationdb/api-c.html#c.fdb_transaction_cancel)\n- For a `t:future/0` or `t:fold_future/0`, [`fdb_future_cancel`](https://apple.github.io/foundationdb/api-c.html#c.fdb_future_cancel)","ref":"erlfdb.html#cancel/2"},{"type":"function","title":"erlfdb.clear/2","doc":"Clears a key in the database.\n\n*C API function*: [`fdb_transaction_clear`](https://apple.github.io/foundationdb/api-c.html#c.fdb_transaction_clear)","ref":"erlfdb.html#clear/2"},{"type":"function","title":"Examples - erlfdb.clear/2","doc":"In a transaction with a single operation:\n\n```erlang\n1> Db = erlfdb:open().\n2> erlfdb:clear(Db, <<\"hello\">>).\n```\n\nIn a transaction with many operations:\n\n```erlang\n1> Db = erlfdb:open().\n2> erlfdb:transactional(Db, fun(Tx) ->\n..     erlfdb:clear(Tx, <<\"hello\">>),\n..     erlfdb:clear(Tx, <<\"foo\">>)\n.. end).\n```","ref":"erlfdb.html#clear/2-examples"},{"type":"function","title":"erlfdb.clear_and_watch/2","doc":"Does both a `clear/2` and `watch/2` in a transaction.","ref":"erlfdb.html#clear_and_watch/2"},{"type":"function","title":"erlfdb.clear_range/3","doc":"Clears a range of keys in the database.\n\nReminder: FoundationDB keys are ordered such that a single operation can work on many keys, as long as they are in a\ncontiguous range.\n\n*C API function*: [`fdb_transaction_clear_range`](https://apple.github.io/foundationdb/api-c.html#c.fdb_transaction_clear_range)","ref":"erlfdb.html#clear_range/3"},{"type":"function","title":"Examples - erlfdb.clear_range/3","doc":"Clears all keys with prefix `<<\"user/0\">>` In a transaction with a single operation:\n\n```erlang\n1> Db = erlfdb:open().\n2> erlfdb:clear_range(Db, <<\"user/0\">>, <<\"user/1\">>).\n```\n\n(!!) Clears all keys in the entire database (!!):\n\n```erlang\n1> Db = erlfdb:open().\n2> erlfdb:clear_range(Db, <<>>, <<16#FF>>).\n```","ref":"erlfdb.html#clear_range/3-examples"},{"type":"function","title":"erlfdb.clear_range_startswith/2","doc":"Equivalent to `clear_range(DbOrTx, Prefix, erlfdb_key:strinc(Prefix)).`\n\n*C API function*: [`fdb_transaction_clear_range`](https://apple.github.io/foundationdb/api-c.html#c.fdb_transaction_clear_range)","ref":"erlfdb.html#clear_range_startswith/2"},{"type":"function","title":"erlfdb.commit/1","doc":"Commits a transaction.\n\n*C API function*: [`fdb_transaction_commit`](https://apple.github.io/foundationdb/api-c.html#c.fdb_transaction_commit)\n\n> #### Tip {: .tip}\n>\n> Use `transactional/2` instead.","ref":"erlfdb.html#commit/1"},{"type":"function","title":"erlfdb.create_database/0","doc":"Creates a database object, which serves as a handle to the FoundationDB server, using the [default cluster file](https://apple.github.io/foundationdb/administration.html#default-cluster-file).\n\n*C API function*: [`fdb_create_database`](https://apple.github.io/foundationdb/api-c.html#c.fdb_create_database)","ref":"erlfdb.html#create_database/0"},{"type":"function","title":"erlfdb.create_database/1","doc":"Creates a database object, which serves as a handle to the FoundationDB server, using the provided cluster file string.\n\n*C API function*: [`fdb_create_database`](https://apple.github.io/foundationdb/api-c.html#c.fdb_create_database)","ref":"erlfdb.html#create_database/1"},{"type":"function","title":"erlfdb.create_transaction/1","doc":"Creates a transaction from a database.\n\nThe caller is responsible for calling\n`commit/1` when appropriate and implementing any retry behavior.\n\n*C API function*: [`fdb_database_create_transaction`](https://apple.github.io/foundationdb/api-c.html#c.fdb_database_create_transaction)\n\n> #### Tip {: .tip}\n>\n> Use `transactional/2` instead.","ref":"erlfdb.html#create_transaction/1"},{"type":"function","title":"erlfdb.error_predicate/2","doc":"Evaluates a predicate against an error code.\n\n*C API function*: [`fdb_error_predicate`](https://apple.github.io/foundationdb/api-c.html#c.fdb_error_predicate)","ref":"erlfdb.html#error_predicate/2"},{"type":"function","title":"erlfdb.fold_mapped_range_future/4","doc":"Deprecated. Equivalent to `fold_mapped_range_future(Tx, StartKey, EndKey, Mapper, [])`.","ref":"erlfdb.html#fold_mapped_range_future/4"},{"type":"function","title":"erlfdb.fold_range/5","doc":"Equivalent to `fold_range(DbOrTex, StartKey, EndKey, Fun, [])`.","ref":"erlfdb.html#fold_range/5"},{"type":"function","title":"erlfdb.fold_range/6","doc":"Gets a range of key-value pairs. Executes a reducing function as the pairs are retrieved.\n\n> #### Tip {: .tip}\n>\n> Use `get_range/5` with `wait_for_all_interleaving/2` instead.","ref":"erlfdb.html#fold_range/6"},{"type":"function","title":"erlfdb.fold_range_future/4","doc":"Deprecated. Gets a `t:fold_future/0` from a key range.\n\n> #### Tip {: .tip}\n>\n> Use `get_range/5` with option `[{wait, false}]` instead.","ref":"erlfdb.html#fold_range_future/4"},{"type":"function","title":"erlfdb.fold_range_wait/4","doc":"Deprecated. Equivalent to `fold_range_wait(Tx, FF, Fun, Acc, [])`.","ref":"erlfdb.html#fold_range_wait/4"},{"type":"function","title":"erlfdb.fold_range_wait/5","doc":"Retrieves results from a `t:fold_future/0`. Executes a reducing function as the pairs are retrieved.\n\n> #### Tip {: .tip}\n>\n> Use `wait_for_all_interleaving/2` instead.","ref":"erlfdb.html#fold_range_wait/5"},{"type":"function","title":"erlfdb.get/1","doc":"Returns the result of a ready future.\n\n*C API functions*: [`fdb_future_get_*`](https://apple.github.io/foundationdb/api-c.html#c.fdb_future_get_int64)\n\n> #### Tip {: .tip}\n>\n> Use one of the `wait*` functions instead, such as `wait/1`.","ref":"erlfdb.html#get/1"},{"type":"function","title":"erlfdb.get/2","doc":"Gets a value from the database.\n\nWhen used inside a `transactional/2`, the return is a `t:future/0`.\n\n*C API function*: [`fdb_transaction_get`](https://apple.github.io/foundationdb/api-c.html#c.fdb_transaction_get)","ref":"erlfdb.html#get/2"},{"type":"function","title":"Examples - erlfdb.get/2","doc":"Using a `t:database/0` as input, the value is retrieved:\n\n```erlang\n1> Db = erlfdb:open().\n3> <<\"world\">> = erlfdb:get(Db, <<\"hello\">>).\n```\n\nUsing a `t:transaction/0` as input, you must wait on the `t:future/0`:\n\n```erlang\n1> Db = erlfdb:open().\n2> erlfdb:transactional(Db, fun(Tx) ->\n..     Future = erlfdb:get(Tx, <<\"hello\">>),\n..     <<\"world\">> = erlfdb:wait(Future)\n.. end).\n```","ref":"erlfdb.html#get/2-examples"},{"type":"function","title":"erlfdb.get_addresses_for_key/2","doc":"Returns a list of public network addresses as strings, one for each of the storage servers responsible for storing key_name and its associated value.\n\n*C API function*: [`fdb_transaction_get_addresses_for_key`](https://apple.github.io/foundationdb/api-c.html#c.fdb_transaction_get_addresses_for_key)","ref":"erlfdb.html#get_addresses_for_key/2"},{"type":"function","title":"erlfdb.get_and_watch/2","doc":"Does both a `get/2` and `watch/2` in a transaction.","ref":"erlfdb.html#get_and_watch/2"},{"type":"function","title":"erlfdb.get_approximate_size/1","doc":"Gets the approximate transaction size so far.\n\n*C API function*: [`fdb_transaction_get_approximate_size`](https://apple.github.io/foundationdb/api-c.html#c.fdb_transaction_get_approximate_size)","ref":"erlfdb.html#get_approximate_size/1"},{"type":"function","title":"erlfdb.get_committed_version/1","doc":"Retrieves the database version number at which a given transaction was committed.\n\nThis is not needed in simple cases. Not compatible with `transactional/2`.\n\n*C API function*: [`fdb_transaction_get_committed_version`](https://apple.github.io/foundationdb/api-c.html#c.fdb_transaction_get_committed_version)","ref":"erlfdb.html#get_committed_version/1"},{"type":"function","title":"erlfdb.get_conflicting_keys/1","doc":"Gets any conflicting keys on this transaction.","ref":"erlfdb.html#get_conflicting_keys/1"},{"type":"function","title":"erlfdb.get_error/1","doc":"Returns `ok` if future is ready and not in an error state, and an `t:error/0` otherwise.\n\n*C API function*: [`fdb_future_get_error`](https://apple.github.io/foundationdb/api-c.html#c.fdb_future_get_error)\n\n> #### Tip {: .tip}\n>\n> Use one of the `wait*` functions instead, such as `wait/1`.","ref":"erlfdb.html#get_error/1"},{"type":"function","title":"erlfdb.get_error_string/1","doc":"Returns a (somewhat) human-readable English message from an error code.\n\n*C API function*: [`fdb_get_error`](https://apple.github.io/foundationdb/api-c.html#c.fdb_get_error)","ref":"erlfdb.html#get_error_string/1"},{"type":"function","title":"erlfdb.get_estimated_range_size/3","doc":"Returns an estimated byte size of the key range.\n\n*C API function*: [`fdb_transaction_get_estimated_range_size_bytes`](https://apple.github.io/foundationdb/api-c.html#c.fdb_transaction_get_estimated_range_size_bytes)","ref":"erlfdb.html#get_estimated_range_size/3"},{"type":"function","title":"erlfdb.get_key/2","doc":"Resolves a `t:key_selector/0` against the keys in the database.\n\n*C API function*: [`fdb_transaction_get_key`](https://apple.github.io/foundationdb/api-c.html#c.fdb_transaction_get_key)","ref":"erlfdb.html#get_key/2"},{"type":"function","title":"erlfdb.get_key_ss/2","doc":"With snapshot isolation, resolves a `t:key_selector/0` against the keys in the database.\n\n*C API function*: [`fdb_transaction_get_key`](https://apple.github.io/foundationdb/api-c.html#c.fdb_transaction_get_key)","ref":"erlfdb.html#get_key_ss/2"},{"type":"function","title":"erlfdb.get_last_error/0","doc":"Gets the last error stored by erlfdb in the process dictionary.","ref":"erlfdb.html#get_last_error/0"},{"type":"function","title":"erlfdb.get_mapped_range/4","doc":"Equivalent to `get_mapped_range(Tx, StartKey, EndKey, []).`\n\nThis function never returns a `t:fold_future/0`. Use `[{wait, false}]` with `get_mapped_range/5` if you\nwant future semantics.","ref":"erlfdb.html#get_mapped_range/4"},{"type":"function","title":"erlfdb.get_mapped_range/5","doc":"Experimental. A two-stage GetRange operation that uses the first retrieved list of key-value pairs\nto do a secondary lookup.\n\nTo use this function, any keys or values defined by the mapper MUST be encoded with the FoundationDB Tuple layer.\nUse `m:erlfdb_tuple` for this.\n\nThe `t:mapper/0` is an Erlang tuple that describes the Mapper specification as detailed by\n[Everything about GetMappedRange](https://github.com/apple/foundationdb/wiki/Everything-about-GetMappedRange).\n\n*C API function*: [`fdb_transaction_get_range`](https://apple.github.io/foundationdb/api-c.html#c.fdb_transaction_get_range)","ref":"erlfdb.html#get_mapped_range/5"},{"type":"function","title":"Motivation - erlfdb.get_mapped_range/5","doc":"GetMappedRange was created to support a common pattern when implementing indexed lookups. It can allow your Layer to\nfollow a single level of indirection: reading one key and then \"hopping\" over to read the key pointed to by that key.\nInstead of doing so with 2 separate waits in your transaction, GetMappedRange sends the small mapping operation to\nthe FoundationDB server so that it may do it for us. In doing so, we effectively perform 2 chained gets with\na single network round trip between client and server.\n\n[Data Modeling with Indirection](https://apple.github.io/foundationdb/data-modeling.html#indirection)","ref":"erlfdb.html#get_mapped_range/5-motivation"},{"type":"function","title":"Examples - erlfdb.get_mapped_range/5","doc":"Here is a minimal example. First we set up the following key-value pairs in the database:\n\n```\n(a, {b})\n({b}, c)\n```\n\nLet's create our kvs:\n\n```erlang\n1> Db = erlfdb:open().\n2> erlfdb:transactional(Db, fun(Tx) ->\n..    erlfdb:set(Tx, <<\"a\">>, erlfdb_tuple:pack({<<\"b\">>})),\n..    erlfdb:set(Tx, erlfdb_tuple:pack({<<\"b\">>}), <<\"c\">>)\n.. end).\n```\n\nNow, we can retrieve the `<<\"c\">>` value in a single shot:\n\n```erlang\n3> Result = erlfdb:transactional(Tenant, fun(Tx) ->\n..     erlfdb:get_mapped_range(\n..         Tx,\n..         <<\"a\">>,\n..         erlfdb_key:strinc(<<\"a\">>),\n..         {<<\"{V[0]}\">>, <<\"{...}\">>},\n..         []\n..     )\n.. end).\n```\n\nThe `Result` includes **both** of our setup key-value pairs using only a single network round-trip to\nthe FoundationDB server.","ref":"erlfdb.html#get_mapped_range/5-examples"},{"type":"function","title":"erlfdb.get_next_tx_id/1","doc":"Gets a local auto-incrementing integer for this transaction.\n\nEach time this function is executed, a local counter on the transaction is incremented,\nand the value returned.\n\nThis integer is useful in combination with versionstamps to ensure uniqueness\nof versionstamped keys and values for your transaction.","ref":"erlfdb.html#get_next_tx_id/1"},{"type":"function","title":"Examples - erlfdb.get_next_tx_id/1","doc":"```erlang\n1> erlfdb:transactional(Db, fun(Tx) ->\n..\n..   Key1 = erlfdb_tuple:pack_vs({<<\"sample\">>,\n..          {versionstamp, 16#ffffffffffffffff, 16#ffff, erlfdb:get_next_tx_id(Tx)}}),\n..   erlfdb:set_versionstamped_key(Key1, <<\"hello\">>),\n..\n..   Key2 = erlfdb_tuple:pack_vs({<<\"sample\">>,\n..          {versionstamp, 16#ffffffffffffffff, 16#ffff, erlfdb:get_next_tx_id(Tx)}}),\n..   erlfdb:set_versionstamped_key(Key2, <<\"world\">>)\n..\n.. end)\n```\n\nWithout `get_next_tx_id`, the versionstamped keys generated at commit time would collide, because\nexactly one versionstamp is created at commit time.","ref":"erlfdb.html#get_next_tx_id/1-examples"},{"type":"function","title":"erlfdb.get_range/3","doc":"Gets a range of key-value pairs from the database.\n\nThis function never returns a `t:fold_future/0`. Use `[{wait, false}]` with `get_raange/4` if you\nwant future semantics.\n\n*C API function*: [`fdb_transaction_get_range`](https://apple.github.io/foundationdb/api-c.html#c.fdb_transaction_get_range)","ref":"erlfdb.html#get_range/3"},{"type":"function","title":"Examples - erlfdb.get_range/3","doc":"Gets all key-value pairs that are between keys `<<\"user/0\">>` and `<<\"user/1\">>`. No key with prefix `<<\"user/1\">>` will be\nreturned. (right side exclusive)\n\n```erlang\n1> Db = erlfdb:open().\n2> erlfdb:get_range(DB, <<\"user/0\">>, <<\"user/1\">>).\n```","ref":"erlfdb.html#get_range/3-examples"},{"type":"function","title":"erlfdb.get_range/4","doc":"Gets a range of key-value pairs from the database.\n\n*C API function*: [`fdb_transaction_get_range`](https://apple.github.io/foundationdb/api-c.html#c.fdb_transaction_get_range)","ref":"erlfdb.html#get_range/4"},{"type":"function","title":"Wait option - erlfdb.get_range/4","doc":"A `wait` option can be provided with the following effect:\n\n- When `true`, the result is waited upon and returned.\n- When `false`, a `t:fold_future/0` is returned, and the result can be retrieved with `wait_for_all_interleaving/2`.\n- Experimental: When `interleaving`, `get_range_split_points/4` is used to divide the range and the futures are waited upon with `wait_for_all_interleaving/2`.\n  The flattened result is returned.\n\nDefault is `true`.","ref":"erlfdb.html#get_range/4-wait-option"},{"type":"function","title":"Examples - erlfdb.get_range/4","doc":"Gets all key-value pairs that are between keys `<<\"user/0\">>` and `<<\"user/1\">>`. No key with prefix `<<\"user/1\">>` will be\nreturned. (left-inclusive, right-exclusive)\n\n```erlang\n1> Db = erlfdb:open().\n2> erlfdb:transactional(Db, fun(Tx) ->\n..     Future = erlfdb:get_range(Tx, <<\"user/0\">>, <<\"user/1\">>, [{wait, false}]),\n..     [Result] = erlfdb:wait_for_all_interleaving(Tx, [Future]),\n..     Result\n.. end).\n```","ref":"erlfdb.html#get_range/4-examples"},{"type":"function","title":"erlfdb.get_range_split_points/4","doc":"Returns a list of keys that can split the given range into (roughly) equally sized chunks based on chunk_size.\n\n*C API function*: [`fdb_transaction_get_range_split_points`](https://apple.github.io/foundationdb/api-c.html#c.fdb_transaction_get_range_split_points)","ref":"erlfdb.html#get_range_split_points/4"},{"type":"function","title":"erlfdb.get_range_startswith/2","doc":"Gets a range of key-value pairs from the database that begin with the given prefix.\n\nThis function never returns a `t:fold_future/0`. If you want future semantics, use `[{wait, false}]`\nwith `get_raange_startswith/3`.\n\n*C API function*: [`fdb_transaction_get_range`](https://apple.github.io/foundationdb/api-c.html#c.fdb_transaction_get_range)","ref":"erlfdb.html#get_range_startswith/2"},{"type":"function","title":"erlfdb.get_range_startswith/3","doc":"Equivalent to `get_range(Tx, Prefix, erlfdb_key:strinc(Prefix), Options).`","ref":"erlfdb.html#get_range_startswith/3"},{"type":"function","title":"erlfdb.get_read_version/1","doc":"Gets the snapshot read version used by a transaction.\n\nThis is not needed in simple cases.\n\n*C API function*: [`fdb_transaction_get_read_version`](https://apple.github.io/foundationdb/api-c.html#c.fdb_transaction_get_read_version)","ref":"erlfdb.html#get_read_version/1"},{"type":"function","title":"erlfdb.get_ss/2","doc":"With snapshot isolation, gets a value from the database.\n\n*C API function*: [`fdb_transaction_get`](https://apple.github.io/foundationdb/api-c.html#c.fdb_transaction_get)","ref":"erlfdb.html#get_ss/2"},{"type":"function","title":"erlfdb.get_versionstamp/1","doc":"Gets the versionstamp value that was used by any versionstamp operations in this transaction.\n\nThis is not needed in simple cases.\n\n*C API function*: [`fdb_transaction_get_versionstamp`](https://apple.github.io/foundationdb/api-c.html#c.fdb_transaction_get_versionstamp)","ref":"erlfdb.html#get_versionstamp/1"},{"type":"function","title":"erlfdb.get_writes_allowed/1","doc":"Returns false if `disallow_writes` was set on the transaction.","ref":"erlfdb.html#get_writes_allowed/1"},{"type":"function","title":"erlfdb.has_watches/1","doc":"Returns true if at least one operation on the transaction up until now has been a watch.","ref":"erlfdb.html#has_watches/1"},{"type":"function","title":"erlfdb.is_read_only/1","doc":"Returns true if all operations on the transaction up until now have been reads.","ref":"erlfdb.html#is_read_only/1"},{"type":"function","title":"erlfdb.is_ready/1","doc":"Returns `true` if the future is ready. That is, its result can be retrieved\nimmediately.\n\n*C API function*: [`fdb_future_is_ready`](https://apple.github.io/foundationdb/api-c.html#c.fdb_future_is_ready)\n\n> #### Tip {: .tip}\n>\n> Use one of the `wait*` functions instead, such as `wait/1`.","ref":"erlfdb.html#is_ready/1"},{"type":"function","title":"erlfdb.max/3","doc":"Performs an atomic maximum on the value in the database.\n\n*C API function*: [`fdb_transaction_atomic_op`](https://apple.github.io/foundationdb/api-c.html#c.fdb_transaction_atomic_op)\n\nSee `FDBMutationType.FDB_MUTATION_TYPE_MAX`.","ref":"erlfdb.html#max/3"},{"type":"function","title":"erlfdb.min/3","doc":"Performs an atomic minimum on the value in the database.\n\n*C API function*: [`fdb_transaction_atomic_op`](https://apple.github.io/foundationdb/api-c.html#c.fdb_transaction_atomic_op)\n\nSee `FDBMutationType.FDB_MUTATION_TYPE_MIN`.","ref":"erlfdb.html#min/3"},{"type":"function","title":"erlfdb.on_error/2","doc":"Implements the recommended retry and backoff behavior for a transaction.\n\n*C API function*: [`fdb_transaction_on_error`](https://apple.github.io/foundationdb/api-c.html#c.fdb_transaction_on_error)\n\n> #### Tip {: .tip}\n>\n> Use `transactional/2` instead.","ref":"erlfdb.html#on_error/2"},{"type":"function","title":"erlfdb.open/0","doc":"Opens a database using the [default cluster file](https://apple.github.io/foundationdb/administration.html#default-cluster-file).\n\nSee `open/2` for more.","ref":"erlfdb.html#open/0"},{"type":"function","title":"erlfdb.open/1","doc":"Opens a database using the provided cluster file string.\n\nSee `open/2` for more.","ref":"erlfdb.html#open/1"},{"type":"function","title":"erlfdb.open/2","doc":"Opens a database using the provided cluster file string.\n\n*C API function*: [`fdb_create_database`](https://apple.github.io/foundationdb/api-c.html#c.fdb_create_database)\n\n`open/2` will lazily create and store a number of database objects based on the configuration\noptions provided. For optimal performance, your application should call `open/2` when a `t:database/0` object is\nneeded. Doing so will ensure even distribution of work among client threads when using the\n[`client_threads_per_version`](thread-design.html) network option.","ref":"erlfdb.html#open/2"},{"type":"function","title":"Options - erlfdb.open/2","doc":"- `dist`: Provides a work distribution stategy, which defaults to `scheduler_id`.","ref":"erlfdb.html#open/2-options"},{"type":"function","title":"Work distribution with the `dist` option - erlfdb.open/2","doc":"It's usually desirable for the application to create a certain number of database objects that can be re-used\nacross many transactions. The `dist` option provides some common strategies for doing so. If you want to implement\nyour own, use `create_database/1` directly instead.\n\n  - `scheduler_id`: (default) A new database object will be created for each scheduler id in the running system for each unique\n    cluster file.  This strategy does not claim that a particular item of work will remain on a scheduler -- only\n    that the scheduler_id itself is helpful in distributing work in a roughly even fashion.\n  - `cluster_file`: A new database object will be created for each unique cluster file provided. This option is not\n    compatible with `client_threads_per_version` > 1.","ref":"erlfdb.html#open/2-work-distribution-with-the-dist-option"},{"type":"function","title":"erlfdb.open_all/0","doc":"Creates database objects equal to the number of online schedulers, using the\n[default cluster file](https://apple.github.io/foundationdb/administration.html#default-cluster-file).\n\nSee `open_all/2` for more.","ref":"erlfdb.html#open_all/0"},{"type":"function","title":"erlfdb.open_all/1","doc":"Creates database objects equal to the number of online schedulers.\n\nSee `open_all/2` for more.","ref":"erlfdb.html#open_all/1"},{"type":"function","title":"erlfdb.open_all/2","doc":"Creates all database objects according to the work distribution strategy.\n\n*C API function*: [`fdb_create_database`](https://apple.github.io/foundationdb/api-c.html#c.fdb_create_database)\n\n**Why is this useful?** Due to the use of `m:persistent_term` by `open/2`, it can be beneficial to create all database\nobjects shortly after startup, when the runtime system is fresh and the number of processes is low. Calling this function\nduring your application start-up will ensure that you are in compliance with the [Best Practices for Using Persistent Terms](https://www.erlang.org/doc/apps/erts/persistent_term.html#module-best-practices-for-using-persistent-terms).","ref":"erlfdb.html#open_all/2"},{"type":"function","title":"erlfdb.open_tenant/2","doc":"Opens a handle to the [Tenant](https://apple.github.io/foundationdb/tenants.html).\n\n*C API function*: [`fdb_database_open_tenant`](https://apple.github.io/foundationdb/api-c.html#c.fdb_database_open_tenant)","ref":"erlfdb.html#open_tenant/2"},{"type":"function","title":"erlfdb.reset/1","doc":"Resets a transaction.\n\n*C API function*: [`fdb_transaction_reset`](https://apple.github.io/foundationdb/api-c.html#c.fdb_transaction_reset)\n\n> #### Tip {: .tip}\n>\n> Use `transactional/2` instead.","ref":"erlfdb.html#reset/1"},{"type":"function","title":"erlfdb.set/3","doc":"Sets a key to the given value.\n\n*C API function*: [`fdb_transaction_set`](https://apple.github.io/foundationdb/api-c.html#c.fdb_transaction_set)","ref":"erlfdb.html#set/3"},{"type":"function","title":"Examples - erlfdb.set/3","doc":"In a transaction with a single operation:\n\n```erlang\n1> Db = erlfdb:open().\n2> erlfdb:set(Db, <<\"hello\">>, <<\"world\">>).\n```\n\nIn a transaction with many operations:\n\n```erlang\n1> Db = erlfdb:open().\n2> erlfdb:transactional(Db, fun(Tx) ->\n..     erlfdb:set(Tx, <<\"hello\">>, <<\"world\">>),\n..     erlfdb:set(Tx, <<\"foo\">>, <<\"bar\">>)\n.. end).\n```","ref":"erlfdb.html#set/3-examples"},{"type":"function","title":"erlfdb.set_and_watch/3","doc":"Does both a `set/3` and `watch/2` in a transaction.","ref":"erlfdb.html#set_and_watch/3"},{"type":"function","title":"erlfdb.set_option/2","doc":"Equivalent to `set_option(DbOrTx, Option, <<>>)`.","ref":"erlfdb.html#set_option/2"},{"type":"function","title":"erlfdb.set_option/3","doc":"Sets a valued option on a database or transaction.\n\n*C API functions*:\n\n- For a `t:database/0`, [`fdb_database_set_option](https://apple.github.io/foundationdb/api-c.html#c.fdb_database_set_option)\n- For a `t:transaction/0, [`fdb_transaction_set_option`](https://apple.github.io/foundationdb/api-c.html#c.fdb_transaction_set_option)","ref":"erlfdb.html#set_option/3"},{"type":"function","title":"erlfdb.set_read_version/2","doc":"Sets the snapshot read version used by a transaction.\n\nThis is not needed in simple cases.\n\n*C API function*: [`fdb_transaction_set_read_version`](https://apple.github.io/foundationdb/api-c.html#c.fdb_transaction_set_read_version)","ref":"erlfdb.html#set_read_version/2"},{"type":"function","title":"erlfdb.set_versionstamped_key/3","doc":"Transforms key using a versionstamp for the transaction.\n\n*C API function*: [`fdb_transaction_atomic_op`](https://apple.github.io/foundationdb/api-c.html#c.fdb_transaction_atomic_op)\n\nSee `FDBMutationType.FDB_MUTATION_TYPE_SET_VERSIONSTAMPED_KEY`.","ref":"erlfdb.html#set_versionstamped_key/3"},{"type":"function","title":"erlfdb.set_versionstamped_value/3","doc":"Transforms param using a versionstamp for the transaction.\n\n*C API function*: [`fdb_transaction_atomic_op`](https://apple.github.io/foundationdb/api-c.html#c.fdb_transaction_atomic_op)\n\nSee `FDBMutationType.FDB_MUTATION_TYPE_SET_VERSIONSTAMPED_VALUE`.","ref":"erlfdb.html#set_versionstamped_value/3"},{"type":"function","title":"erlfdb.snapshot/1","doc":"Use this to modify a transaction so that operations on the returned object are\nperformed as [Snapshot Reads](https://apple.github.io/foundationdb/developer-guide.html#snapshot-reads).","ref":"erlfdb.html#snapshot/1"},{"type":"function","title":"Example - erlfdb.snapshot/1","doc":"```erlang\n1> Db = erlfdb:open().\n2> erlfdb:transactional(Db, fun(Tx) ->\n..     SS = erlfdb:snapshot(Tx),\n..     F1 = erlfdb:get(Tx, <<\"strictly serializable read\">>),\n..     F2 = erlfdb:get(SS, <<\"snapshot isolation read\">>),\n..     erlfdb:wait_for_all([F1, F2])\n.. end).\n```","ref":"erlfdb.html#snapshot/1-example"},{"type":"function","title":"erlfdb.tenant_create_transaction/1","doc":"Creates a transaction from an open tenant.\n\nThe caller is responsible for calling\n`commit/` when appropriate and implementing any retry behavior.\n\n*C API function*: [`fdb_tenant_create_transaction`](https://apple.github.io/foundationdb/api-c.html#c.fdb_tenant_create_transaction)\n\n> #### Tip {: .tip}\n>\n> Use `transactional/2` instead.","ref":"erlfdb.html#tenant_create_transaction/1"},{"type":"function","title":"erlfdb.transactional/2","doc":"Executes the provided 1-arity function in a transaction.\n\nYou'll use the other erlfdb functions inside a transaction to safely achieve interesting\nbehavior in your application. The [FoundationDB Transaction Manifesto](https://apple.github.io/foundationdb/transaction-manifesto.html)\nis a good starting point for understanding the why and how. Some examples are below.","ref":"erlfdb.html#transactional/2"},{"type":"function","title":"Transaction Commit and Error Handling - erlfdb.transactional/2","doc":"The provided 1-arity function is executed, and when it returns, erlfdb inspects the transaction is\nfor its read-only status. If read-only, control returns to the caller immediately.\nOtherwise, the transaction is committed to the database with `commit/1`.\nIn both cases, the return value from the provided function is returned to\nthe caller after the transaction has been completed.\n\nAn error can be encountered during the transaction (for example, when a key conflict is detected by the FDB Server).\nIf it is an error code that FoundationDB recommends as eligible for retry, your function will be executed\nagain. For this reason, **your function should have no side-effects**. Logging, sending messages,\nand updates to an ets table that exist in your transaction function will execute multiple times when a key\nconflict is encountered. Moreover, if your transaction has any branching logic, such side effects can be misleading or harmful.\n\nImagine a transaction to be a pure function that receives the entire database as input and returns the entire database\nas output. If your function's implementation is [purely functional](https://en.wikipedia.org/wiki/Purely_functional_programming) under\nthis point of view, then it is safe.\n\nThe transaction will be executed repeatedly as needed until either (a) it is completed successfully, and control flow returns to the\ncaller, or (b) a non-retriable error is thrown. A transaction timeout is an example of a non-retriable error, but\nby default, the timeout is set to `infinity`.\n\n*C API functions*:\n\n- For the commit, [`fdb_transaction_commit`](https://apple.github.io/foundationdb/api-c.html#c.fdb_transaction_commit).\n- For the retry behavior, [`fdb_transaction_on_error`](https://apple.github.io/foundationdb/api-c.html#c.fdb_transaction_on_error).","ref":"erlfdb.html#transactional/2-transaction-commit-and-error-handling"},{"type":"function","title":"Arguments - erlfdb.transactional/2","doc":"* `Db` / `Tenant`: The context under which the transaction is created.\n  * `UserFun`: 1-arity function to be executed with FDB transactional\n    semantics. The transaction (`t:transaction/0`) is provided as the argument to the function.","ref":"erlfdb.html#transactional/2-arguments"},{"type":"function","title":"Examples - erlfdb.transactional/2","doc":"Using a database as the transactional context:\n\n```erlang\n1> Db = erlfdb:open().\n2> erlfdb:transactional(Db, fun(Tx) ->\n..     not_found = erlfdb:wait(erlfdb:get(Tx, <<\"hello\">>)),\n..     erlfdb:set(Tx, <<\"hello\">>, <<\"world\">>)\n.. end).\n```\n\nUsing a tenant as the transactional context:\n\n```erlang\n1> Db = erlfdb:open().\n2> Tenant = erlfdb:open_tenant(Db, <<\"some-org\">>)\n2> erlfdb:transactional(Tenant, fun(Tx) ->\n..     not_found = erlfdb:wait(erlfdb:get(Tx, <<\"hello\">>)),\n..     erlfdb:set(Tx, <<\"hello\">>, <<\"world\">>)\n.. end).\n```\n\nPerforming non-trivial logic inside a transaction:\n\n```erlang\n1> Db = erlfdb:open().\n2> erlfdb:set(Db, <<\"toggle\">>, <<\"off\">>).\n3> erlfdb:transactional(Db, fun(Tx) ->\n..     case erlfdb:wait(erlfdb:get(Tx, <<\"toggle\">>)) of\n..         <<\"off\">> ->\n..             erlfdb:set(Tx, <<\"toggle\">>, <<\"on\">>),\n..             off_to_on;\n..         <<\"on\">> ->\n..             erlfdb:set(Tx, <<\"toggle\">>, <<\"off\">>),\n..             on_to_off\n..     end\n.. end).\n```","ref":"erlfdb.html#transactional/2-examples"},{"type":"function","title":"erlfdb.wait/1","doc":"Equivalent to `wait(Future, [])`.","ref":"erlfdb.html#wait/1"},{"type":"function","title":"erlfdb.wait/2","doc":"Waits for future to be ready, and returns the result.\n\nIf first argument is not a future, then simply return the first argument.","ref":"erlfdb.html#wait/2"},{"type":"function","title":"Examples - erlfdb.wait/2","doc":"`wait/2` is typically used inside a transaction:\n\n```erlang\n1> Db = erlfdb:open().\n2> erlfdb:transactional(Db, fun(Tx) ->\n..     erlfdb:wait(erlfdb:get(Tx, <<\"hello\">>)),\n.. end).\n```","ref":"erlfdb.html#wait/2-examples"},{"type":"function","title":"erlfdb.wait_for_all/1","doc":"Equivalent to `wait_for_all(Futures, [])`.","ref":"erlfdb.html#wait_for_all/1"},{"type":"function","title":"erlfdb.wait_for_all/2","doc":"Waits for each future and returns all results. The order of the results\nis guaranteed to match the order of the futures.","ref":"erlfdb.html#wait_for_all/2"},{"type":"function","title":"Examples - erlfdb.wait_for_all/2","doc":"`wait_for_all/2` returns the results from all futures:\n\n```erlang\n1> Db = erlfdb:open().\n2> erlfdb:transactional(Db, fun(Tx) ->\n..     T = erlfdb:get(Tx, <<\"tortoise\">>)),\n..     H = erlfdb:get(Tx, <<\"hare\">>)),\n..     erlfdb:wait_for_all([T, H])\n.. end).\n```","ref":"erlfdb.html#wait_for_all/2-examples"},{"type":"function","title":"erlfdb.wait_for_all_interleaving/2","doc":"Equivalent to `wait_for_all_interleaving(Tx, Futures, [])`.","ref":"erlfdb.html#wait_for_all_interleaving/2"},{"type":"function","title":"erlfdb.wait_for_all_interleaving/3","doc":"Experimental. Waits for all futures and returns all results. The order of the results\nis guaranteed to match the order of the futures. Uses an interleaving approach,\nwhich is described below.\n\nA future of type `t:fold_future/0` may require multiple round trips to the database\nserver. When provided more than one, the requests to the database\nare pipelined so that the server can optimize retrieval of keys. In this interleaving\napproach, the client sends all known requests to the server as quickly as possible.\n\nThis is an experimental optimization of `wait_for_all/1`, and\nis functionally equivalent.","ref":"erlfdb.html#wait_for_all_interleaving/3"},{"type":"function","title":"Examples - erlfdb.wait_for_all_interleaving/3","doc":"Imagine a large set of users stored with prefix `<<\"user/\">>` and a large set of blog\nposts stored with prefix `<<\"post/\">>`. We can retrieve all users and posts\nefficiently with `wait_for_all_interleaving/2`:\n\n```erlang\n1> Db = erlfdb:open().\n2> erlfdb:transactional(Db, fun(Tx) ->\n..     UsersF = erlfdb:get_range_startswith(Tx, <<\"user/\">>, [{wait, false}])),\n..     PostsF = erlfdb:get_range_startswith(Tx, <<\"post/\">>, [{wait, false}])),\n..     erlfdb:wait_for_all_interleaving(Tx, [UsersF, PostsF])\n.. end).\n```","ref":"erlfdb.html#wait_for_all_interleaving/3-examples"},{"type":"function","title":"erlfdb.wait_for_any/1","doc":"Equivalent to `wait_for_any(Futures, [])`.","ref":"erlfdb.html#wait_for_any/1"},{"type":"function","title":"erlfdb.wait_for_any/2","doc":"Waits for one of the provided futures in the list to enter ready state. When\nsuch a future is found, return it. The result of that future must be retrieved\nby the caller with `get/1`.","ref":"erlfdb.html#wait_for_any/2"},{"type":"function","title":"Examples - erlfdb.wait_for_any/2","doc":"`wait_for_any/2` returns a future. It will typically be the future that is the first\nto resolve, but that is not guaranteed:\n\n```erlang\n1> Db = erlfdb:open().\n2> erlfdb:transactional(Db, fun(Tx) ->\n..     T = erlfdb:get(Tx, <<\"tortoise\">>)),\n..     H = erlfdb:get(Tx, <<\"hare\">>)),\n..     Winner = erlfdb:wait_for_any([T, H]),\n..     erlfdb:get(Winner)\n.. end).\n```","ref":"erlfdb.html#wait_for_any/2-examples"},{"type":"function","title":"erlfdb.watch/2","doc":"Creates a watch on a key.\n\n[An Overview how Watches Work](https://github.com/apple/foundationdb/wiki/An-Overview-how-Watches-Work)\n\n*C API function*: [`fdb_transaction_watch`](https://apple.github.io/foundationdb/api-c.html#c.fdb_transaction_watch)","ref":"erlfdb.html#watch/2"},{"type":"function","title":"Examples - erlfdb.watch/2","doc":"In this example, when `wait/1` returns, the value at the key `<<\"hello\">>` has changed. We must then do a `get/2` if we\nwant to know what the value is.\n\n```erlang\n1> Db = erlfdb:open().\n2> Watch = erlfdb:watch(Db, <<\"hello\">>).\n3> ok = erlfdb:wait(Watch).\n4> erlfdb:get(Db, <<\"hello\">>).\n```\n\nIn this example, we rely on the future's `ready` message to be delivered to our calling process. This form of the\nready message (`t:watch_future_ready_message/0`) is valid only for futures created by `watch/2`.\n\n```erlang\n1> Db = erlfdb:open(),\n2> Watch = erlfdb:watch(Db, <<\"hello\">>).\n3> receive\n..    {_Ref, ready} ->\n..        erlfdb:get(Db, <<\"hello\">>)\n.. end\n```","ref":"erlfdb.html#watch/2-examples"},{"type":"type","title":"erlfdb.atomic_mode/0","doc":"","ref":"erlfdb.html#t:atomic_mode/0"},{"type":"type","title":"erlfdb.atomic_operand/0","doc":"","ref":"erlfdb.html#t:atomic_operand/0"},{"type":"type","title":"erlfdb.cluster_filename/0","doc":"","ref":"erlfdb.html#t:cluster_filename/0"},{"type":"type","title":"erlfdb.database/0","doc":"","ref":"erlfdb.html#t:database/0"},{"type":"type","title":"erlfdb.database_option/0","doc":"","ref":"erlfdb.html#t:database_option/0"},{"type":"type","title":"erlfdb.error/0","doc":"","ref":"erlfdb.html#t:error/0"},{"type":"type","title":"erlfdb.error_predicate/0","doc":"","ref":"erlfdb.html#t:error_predicate/0"},{"type":"type","title":"erlfdb.fold_future/0","doc":"","ref":"erlfdb.html#t:fold_future/0"},{"type":"type","title":"erlfdb.fold_option/0","doc":"","ref":"erlfdb.html#t:fold_option/0"},{"type":"type","title":"erlfdb.future/0","doc":"","ref":"erlfdb.html#t:future/0"},{"type":"type","title":"erlfdb.future_ready_message/0","doc":"","ref":"erlfdb.html#t:future_ready_message/0"},{"type":"type","title":"erlfdb.key/0","doc":"","ref":"erlfdb.html#t:key/0"},{"type":"type","title":"erlfdb.key_selector/0","doc":"","ref":"erlfdb.html#t:key_selector/0"},{"type":"type","title":"erlfdb.kv/0","doc":"","ref":"erlfdb.html#t:kv/0"},{"type":"type","title":"erlfdb.mapped_kv/0","doc":"","ref":"erlfdb.html#t:mapped_kv/0"},{"type":"type","title":"erlfdb.mapper/0","doc":"","ref":"erlfdb.html#t:mapper/0"},{"type":"type","title":"erlfdb.open_option/0","doc":"","ref":"erlfdb.html#t:open_option/0"},{"type":"type","title":"erlfdb.result/0","doc":"","ref":"erlfdb.html#t:result/0"},{"type":"type","title":"erlfdb.snapshot/0","doc":"","ref":"erlfdb.html#t:snapshot/0"},{"type":"type","title":"erlfdb.split_option/0","doc":"","ref":"erlfdb.html#t:split_option/0"},{"type":"type","title":"erlfdb.tenant/0","doc":"","ref":"erlfdb.html#t:tenant/0"},{"type":"type","title":"erlfdb.tenant_name/0","doc":"","ref":"erlfdb.html#t:tenant_name/0"},{"type":"type","title":"erlfdb.transaction/0","doc":"","ref":"erlfdb.html#t:transaction/0"},{"type":"type","title":"erlfdb.transaction_future_ready_message/0","doc":"","ref":"erlfdb.html#t:transaction_future_ready_message/0"},{"type":"type","title":"erlfdb.transaction_option/0","doc":"","ref":"erlfdb.html#t:transaction_option/0"},{"type":"type","title":"erlfdb.value/0","doc":"","ref":"erlfdb.html#t:value/0"},{"type":"type","title":"erlfdb.version/0","doc":"","ref":"erlfdb.html#t:version/0"},{"type":"type","title":"erlfdb.wait_option/0","doc":"","ref":"erlfdb.html#t:wait_option/0"},{"type":"type","title":"erlfdb.watch_future_ready_message/0","doc":"","ref":"erlfdb.html#t:watch_future_ready_message/0"},{"type":"module","title":"erlfdb_directory","doc":"[Directory Layer](https://apple.github.io/foundationdb/developer-guide.html#directories)","ref":"erlfdb_directory.html"},{"type":"function","title":"erlfdb_directory.contains/2","doc":"","ref":"erlfdb_directory.html#contains/2"},{"type":"function","title":"erlfdb_directory.create/3","doc":"","ref":"erlfdb_directory.html#create/3"},{"type":"function","title":"erlfdb_directory.create/4","doc":"","ref":"erlfdb_directory.html#create/4"},{"type":"function","title":"erlfdb_directory.create_or_open/3","doc":"","ref":"erlfdb_directory.html#create_or_open/3"},{"type":"function","title":"erlfdb_directory.create_or_open/4","doc":"","ref":"erlfdb_directory.html#create_or_open/4"},{"type":"function","title":"erlfdb_directory.debug_nodes/2","doc":"","ref":"erlfdb_directory.html#debug_nodes/2"},{"type":"function","title":"erlfdb_directory.exists/2","doc":"","ref":"erlfdb_directory.html#exists/2"},{"type":"function","title":"erlfdb_directory.exists/3","doc":"","ref":"erlfdb_directory.html#exists/3"},{"type":"function","title":"erlfdb_directory.get_id/1","doc":"","ref":"erlfdb_directory.html#get_id/1"},{"type":"function","title":"erlfdb_directory.get_layer/1","doc":"","ref":"erlfdb_directory.html#get_layer/1"},{"type":"function","title":"erlfdb_directory.get_name/1","doc":"","ref":"erlfdb_directory.html#get_name/1"},{"type":"function","title":"erlfdb_directory.get_node_prefix/1","doc":"","ref":"erlfdb_directory.html#get_node_prefix/1"},{"type":"function","title":"erlfdb_directory.get_path/1","doc":"","ref":"erlfdb_directory.html#get_path/1"},{"type":"function","title":"erlfdb_directory.get_root/1","doc":"","ref":"erlfdb_directory.html#get_root/1"},{"type":"function","title":"erlfdb_directory.get_root_for_path/2","doc":"","ref":"erlfdb_directory.html#get_root_for_path/2"},{"type":"function","title":"erlfdb_directory.get_subspace/1","doc":"","ref":"erlfdb_directory.html#get_subspace/1"},{"type":"function","title":"erlfdb_directory.key/1","doc":"","ref":"erlfdb_directory.html#key/1"},{"type":"function","title":"erlfdb_directory.list/2","doc":"","ref":"erlfdb_directory.html#list/2"},{"type":"function","title":"erlfdb_directory.list/3","doc":"","ref":"erlfdb_directory.html#list/3"},{"type":"function","title":"erlfdb_directory.move/4","doc":"","ref":"erlfdb_directory.html#move/4"},{"type":"function","title":"erlfdb_directory.move_to/3","doc":"","ref":"erlfdb_directory.html#move_to/3"},{"type":"function","title":"erlfdb_directory.open/3","doc":"","ref":"erlfdb_directory.html#open/3"},{"type":"function","title":"erlfdb_directory.open/4","doc":"","ref":"erlfdb_directory.html#open/4"},{"type":"function","title":"erlfdb_directory.pack/2","doc":"","ref":"erlfdb_directory.html#pack/2"},{"type":"function","title":"erlfdb_directory.pack_vs/2","doc":"","ref":"erlfdb_directory.html#pack_vs/2"},{"type":"function","title":"erlfdb_directory.range/1","doc":"","ref":"erlfdb_directory.html#range/1"},{"type":"function","title":"erlfdb_directory.range/2","doc":"","ref":"erlfdb_directory.html#range/2"},{"type":"function","title":"erlfdb_directory.remove/2","doc":"","ref":"erlfdb_directory.html#remove/2"},{"type":"function","title":"erlfdb_directory.remove/3","doc":"","ref":"erlfdb_directory.html#remove/3"},{"type":"function","title":"erlfdb_directory.remove_if_exists/2","doc":"","ref":"erlfdb_directory.html#remove_if_exists/2"},{"type":"function","title":"erlfdb_directory.remove_if_exists/3","doc":"","ref":"erlfdb_directory.html#remove_if_exists/3"},{"type":"function","title":"erlfdb_directory.root/0","doc":"","ref":"erlfdb_directory.html#root/0"},{"type":"function","title":"erlfdb_directory.root/1","doc":"","ref":"erlfdb_directory.html#root/1"},{"type":"function","title":"erlfdb_directory.subspace/2","doc":"","ref":"erlfdb_directory.html#subspace/2"},{"type":"function","title":"erlfdb_directory.unpack/2","doc":"","ref":"erlfdb_directory.html#unpack/2"},{"type":"module","title":"erlfdb_float","doc":"Advanced floating point number encoding for `m:erlfdb_tuple`.","ref":"erlfdb_float.html"},{"type":"function","title":"erlfdb_float.decode/1","doc":"","ref":"erlfdb_float.html#decode/1"},{"type":"function","title":"erlfdb_float.encode/1","doc":"","ref":"erlfdb_float.html#encode/1"},{"type":"module","title":"erlfdb_key","doc":"Operations on keys, either by your client Layer, or by the FoundationDB server itself.","ref":"erlfdb_key.html"},{"type":"function","title":"erlfdb_key.first_greater_or_equal/1","doc":"Creates a 'first_greater_or_equal' key selector.\n\n[FDB Developer Guide | Key Selectors](https://apple.github.io/foundationdb/developer-guide.html#key-selectors)","ref":"erlfdb_key.html#first_greater_or_equal/1"},{"type":"function","title":"erlfdb_key.first_greater_than/1","doc":"Creates a 'first_greater_than' key selector.\n\n[FDB Developer Guide | Key Selectors](https://apple.github.io/foundationdb/developer-guide.html#key-selectors)","ref":"erlfdb_key.html#first_greater_than/1"},{"type":"function","title":"erlfdb_key.last_less_or_equal/1","doc":"Creates a 'last_less_or_equal' key selector.\n\n[FDB Developer Guide | Key Selectors](https://apple.github.io/foundationdb/developer-guide.html#key-selectors)","ref":"erlfdb_key.html#last_less_or_equal/1"},{"type":"function","title":"erlfdb_key.last_less_than/1","doc":"Creates a 'last_less_than' key selector.\n\n[FDB Developer Guide | Key Selectors](https://apple.github.io/foundationdb/developer-guide.html#key-selectors)","ref":"erlfdb_key.html#last_less_than/1"},{"type":"function","title":"erlfdb_key.list_to_ranges/1","doc":"Transforms a list of keys into a list of partitioning ranges using the keys as split points.","ref":"erlfdb_key.html#list_to_ranges/1"},{"type":"function","title":"Examples - erlfdb_key.list_to_ranges/1","doc":"```erlang\n1> Keys = [<<\"a\">>, <<\"b\">>, <<\"c\">>].\n2> erlfdb_key:list_to_ranges(Keys).\n[\n    {<<\"a\">>, <<\"b\">>},\n    {<<\"b\">>, <<\"c\">>}\n]\n```","ref":"erlfdb_key.html#list_to_ranges/1-examples"},{"type":"function","title":"erlfdb_key.strinc/1","doc":"Performs a lexocographic increment operation on a binary key.\n\nThe resulting key is the first possible key that is lexicographically ordered after all keys beginning with the input.","ref":"erlfdb_key.html#strinc/1"},{"type":"function","title":"Examples - erlfdb_key.strinc/1","doc":"```erlang\n1> erlfdb_key:strinc(<<0>>).\n<<1>>\n```\n\n```erlang\n1> erlfdb_key:strinc(<<\"hello\">>).\n<<\"hellp\">>\n```\n\n```erlang\n1> erlfdb_key:strinc(<<\"hello\", 16#FF>>).\n<<\"hellp\">>\n```","ref":"erlfdb_key.html#strinc/1-examples"},{"type":"module","title":"erlfdb_nif","doc":"The NIF wrapper around all FoundationDB C API function calls.\n\n> #### Tip {: .tip}\n>\n> Use `m:erlfdb` instead.","ref":"erlfdb_nif.html"},{"type":"function","title":"erlfdb_nif.create_database/1","doc":"","ref":"erlfdb_nif.html#create_database/1"},{"type":"function","title":"erlfdb_nif.database_create_transaction/1","doc":"","ref":"erlfdb_nif.html#database_create_transaction/1"},{"type":"function","title":"erlfdb_nif.database_open_tenant/2","doc":"","ref":"erlfdb_nif.html#database_open_tenant/2"},{"type":"function","title":"erlfdb_nif.database_set_option/2","doc":"","ref":"erlfdb_nif.html#database_set_option/2"},{"type":"function","title":"erlfdb_nif.database_set_option/3","doc":"","ref":"erlfdb_nif.html#database_set_option/3"},{"type":"function","title":"erlfdb_nif.error_predicate/2","doc":"","ref":"erlfdb_nif.html#error_predicate/2"},{"type":"function","title":"erlfdb_nif.future_cancel/1","doc":"","ref":"erlfdb_nif.html#future_cancel/1"},{"type":"function","title":"erlfdb_nif.future_get/1","doc":"","ref":"erlfdb_nif.html#future_get/1"},{"type":"function","title":"erlfdb_nif.future_get_error/1","doc":"","ref":"erlfdb_nif.html#future_get_error/1"},{"type":"function","title":"erlfdb_nif.future_is_ready/1","doc":"","ref":"erlfdb_nif.html#future_is_ready/1"},{"type":"function","title":"erlfdb_nif.future_silence/1","doc":"","ref":"erlfdb_nif.html#future_silence/1"},{"type":"function","title":"erlfdb_nif.get_error/1","doc":"","ref":"erlfdb_nif.html#get_error/1"},{"type":"function","title":"erlfdb_nif.get_max_api_version/0","doc":"","ref":"erlfdb_nif.html#get_max_api_version/0"},{"type":"function","title":"erlfdb_nif.ohai/0","doc":"","ref":"erlfdb_nif.html#ohai/0"},{"type":"function","title":"erlfdb_nif.tenant_create_transaction/1","doc":"","ref":"erlfdb_nif.html#tenant_create_transaction/1"},{"type":"function","title":"erlfdb_nif.transaction_add_conflict_range/4","doc":"","ref":"erlfdb_nif.html#transaction_add_conflict_range/4"},{"type":"function","title":"erlfdb_nif.transaction_atomic_op/4","doc":"","ref":"erlfdb_nif.html#transaction_atomic_op/4"},{"type":"function","title":"erlfdb_nif.transaction_cancel/1","doc":"","ref":"erlfdb_nif.html#transaction_cancel/1"},{"type":"function","title":"erlfdb_nif.transaction_clear/2","doc":"","ref":"erlfdb_nif.html#transaction_clear/2"},{"type":"function","title":"erlfdb_nif.transaction_clear_range/3","doc":"","ref":"erlfdb_nif.html#transaction_clear_range/3"},{"type":"function","title":"erlfdb_nif.transaction_commit/1","doc":"","ref":"erlfdb_nif.html#transaction_commit/1"},{"type":"function","title":"erlfdb_nif.transaction_get/3","doc":"","ref":"erlfdb_nif.html#transaction_get/3"},{"type":"function","title":"erlfdb_nif.transaction_get_addresses_for_key/2","doc":"","ref":"erlfdb_nif.html#transaction_get_addresses_for_key/2"},{"type":"function","title":"erlfdb_nif.transaction_get_approximate_size/1","doc":"","ref":"erlfdb_nif.html#transaction_get_approximate_size/1"},{"type":"function","title":"erlfdb_nif.transaction_get_committed_version/1","doc":"","ref":"erlfdb_nif.html#transaction_get_committed_version/1"},{"type":"function","title":"erlfdb_nif.transaction_get_estimated_range_size/3","doc":"","ref":"erlfdb_nif.html#transaction_get_estimated_range_size/3"},{"type":"function","title":"erlfdb_nif.transaction_get_key/3","doc":"","ref":"erlfdb_nif.html#transaction_get_key/3"},{"type":"function","title":"erlfdb_nif.transaction_get_mapped_range/10","doc":"","ref":"erlfdb_nif.html#transaction_get_mapped_range/10"},{"type":"function","title":"erlfdb_nif.transaction_get_next_tx_id/1","doc":"","ref":"erlfdb_nif.html#transaction_get_next_tx_id/1"},{"type":"function","title":"erlfdb_nif.transaction_get_range/9","doc":"","ref":"erlfdb_nif.html#transaction_get_range/9"},{"type":"function","title":"erlfdb_nif.transaction_get_range_split_points/4","doc":"","ref":"erlfdb_nif.html#transaction_get_range_split_points/4"},{"type":"function","title":"erlfdb_nif.transaction_get_read_version/1","doc":"","ref":"erlfdb_nif.html#transaction_get_read_version/1"},{"type":"function","title":"erlfdb_nif.transaction_get_versionstamp/1","doc":"","ref":"erlfdb_nif.html#transaction_get_versionstamp/1"},{"type":"function","title":"erlfdb_nif.transaction_get_writes_allowed/1","doc":"","ref":"erlfdb_nif.html#transaction_get_writes_allowed/1"},{"type":"function","title":"erlfdb_nif.transaction_has_watches/1","doc":"","ref":"erlfdb_nif.html#transaction_has_watches/1"},{"type":"function","title":"erlfdb_nif.transaction_is_read_only/1","doc":"","ref":"erlfdb_nif.html#transaction_is_read_only/1"},{"type":"function","title":"erlfdb_nif.transaction_on_error/2","doc":"","ref":"erlfdb_nif.html#transaction_on_error/2"},{"type":"function","title":"erlfdb_nif.transaction_reset/1","doc":"","ref":"erlfdb_nif.html#transaction_reset/1"},{"type":"function","title":"erlfdb_nif.transaction_set/3","doc":"","ref":"erlfdb_nif.html#transaction_set/3"},{"type":"function","title":"erlfdb_nif.transaction_set_option/2","doc":"","ref":"erlfdb_nif.html#transaction_set_option/2"},{"type":"function","title":"erlfdb_nif.transaction_set_option/3","doc":"","ref":"erlfdb_nif.html#transaction_set_option/3"},{"type":"function","title":"erlfdb_nif.transaction_set_read_version/2","doc":"","ref":"erlfdb_nif.html#transaction_set_read_version/2"},{"type":"function","title":"erlfdb_nif.transaction_watch/2","doc":"","ref":"erlfdb_nif.html#transaction_watch/2"},{"type":"type","title":"erlfdb_nif.atomic_mode/0","doc":"","ref":"erlfdb_nif.html#t:atomic_mode/0"},{"type":"type","title":"erlfdb_nif.atomic_operand/0","doc":"","ref":"erlfdb_nif.html#t:atomic_operand/0"},{"type":"type","title":"erlfdb_nif.conflict_type/0","doc":"","ref":"erlfdb_nif.html#t:conflict_type/0"},{"type":"type","title":"erlfdb_nif.database/0","doc":"","ref":"erlfdb_nif.html#t:database/0"},{"type":"type","title":"erlfdb_nif.database_option/0","doc":"","ref":"erlfdb_nif.html#t:database_option/0"},{"type":"type","title":"erlfdb_nif.error/0","doc":"","ref":"erlfdb_nif.html#t:error/0"},{"type":"type","title":"erlfdb_nif.error_predicate/0","doc":"","ref":"erlfdb_nif.html#t:error_predicate/0"},{"type":"type","title":"erlfdb_nif.future/0","doc":"","ref":"erlfdb_nif.html#t:future/0"},{"type":"type","title":"erlfdb_nif.future_result/0","doc":"","ref":"erlfdb_nif.html#t:future_result/0"},{"type":"type","title":"erlfdb_nif.key/0","doc":"","ref":"erlfdb_nif.html#t:key/0"},{"type":"type","title":"erlfdb_nif.key_selector/0","doc":"","ref":"erlfdb_nif.html#t:key_selector/0"},{"type":"type","title":"erlfdb_nif.option_value/0","doc":"","ref":"erlfdb_nif.html#t:option_value/0"},{"type":"type","title":"erlfdb_nif.streaming_mode/0","doc":"","ref":"erlfdb_nif.html#t:streaming_mode/0"},{"type":"type","title":"erlfdb_nif.tenant/0","doc":"","ref":"erlfdb_nif.html#t:tenant/0"},{"type":"type","title":"erlfdb_nif.transaction/0","doc":"","ref":"erlfdb_nif.html#t:transaction/0"},{"type":"type","title":"erlfdb_nif.transaction_option/0","doc":"","ref":"erlfdb_nif.html#t:transaction_option/0"},{"type":"type","title":"erlfdb_nif.value/0","doc":"","ref":"erlfdb_nif.html#t:value/0"},{"type":"type","title":"erlfdb_nif.version/0","doc":"","ref":"erlfdb_nif.html#t:version/0"},{"type":"module","title":"erlfdb_sandbox","doc":"Creates a database that is to be used as a sandbox.\n\nThe sandbox database is constructed using a single\nfdbserver process and some default settings are selected.","ref":"erlfdb_sandbox.html"},{"type":"function","title":"erlfdb_sandbox.default_options/0","doc":"Gets the default options for the sandbox database.","ref":"erlfdb_sandbox.html#default_options/0"},{"type":"function","title":"erlfdb_sandbox.open/0","doc":"Opens the sandbox database using the default directory `.erlfdb_sandbox`.","ref":"erlfdb_sandbox.html#open/0"},{"type":"function","title":"erlfdb_sandbox.open/1","doc":"Opens the sandbox database in a subdirectory of\n`.erlfdb_sandbox`, allowing for multiple concurrent sandboxes.","ref":"erlfdb_sandbox.html#open/1"},{"type":"module","title":"erlfdb_subspace","doc":"[Subspace Layer](https://apple.github.io/foundationdb/developer-guide.html#subspaces)","ref":"erlfdb_subspace.html"},{"type":"function","title":"erlfdb_subspace.add/2","doc":"","ref":"erlfdb_subspace.html#add/2"},{"type":"function","title":"erlfdb_subspace.contains/2","doc":"","ref":"erlfdb_subspace.html#contains/2"},{"type":"function","title":"erlfdb_subspace.create/1","doc":"","ref":"erlfdb_subspace.html#create/1"},{"type":"function","title":"erlfdb_subspace.create/2","doc":"","ref":"erlfdb_subspace.html#create/2"},{"type":"function","title":"erlfdb_subspace.key/1","doc":"","ref":"erlfdb_subspace.html#key/1"},{"type":"function","title":"erlfdb_subspace.pack/1","doc":"","ref":"erlfdb_subspace.html#pack/1"},{"type":"function","title":"erlfdb_subspace.pack/2","doc":"","ref":"erlfdb_subspace.html#pack/2"},{"type":"function","title":"erlfdb_subspace.pack_vs/1","doc":"","ref":"erlfdb_subspace.html#pack_vs/1"},{"type":"function","title":"erlfdb_subspace.pack_vs/2","doc":"","ref":"erlfdb_subspace.html#pack_vs/2"},{"type":"function","title":"erlfdb_subspace.range/1","doc":"","ref":"erlfdb_subspace.html#range/1"},{"type":"function","title":"erlfdb_subspace.range/2","doc":"","ref":"erlfdb_subspace.html#range/2"},{"type":"function","title":"erlfdb_subspace.subspace/2","doc":"","ref":"erlfdb_subspace.html#subspace/2"},{"type":"function","title":"erlfdb_subspace.unpack/2","doc":"","ref":"erlfdb_subspace.html#unpack/2"},{"type":"module","title":"erlfdb_tenant_management","doc":"Managing [Tenants](https://apple.github.io/foundationdb/tenants.html)\n\n> #### Warning {: .warning}\n>\n> Tenants are currently experimental and are not recommended for use in production.","ref":"erlfdb_tenant_management.html"},{"type":"function","title":"erlfdb_tenant_management.create_tenant/2","doc":"","ref":"erlfdb_tenant_management.html#create_tenant/2"},{"type":"function","title":"erlfdb_tenant_management.delete_tenant/2","doc":"","ref":"erlfdb_tenant_management.html#delete_tenant/2"},{"type":"function","title":"erlfdb_tenant_management.get_tenant/2","doc":"","ref":"erlfdb_tenant_management.html#get_tenant/2"},{"type":"function","title":"erlfdb_tenant_management.list_tenants/1","doc":"","ref":"erlfdb_tenant_management.html#list_tenants/1"},{"type":"function","title":"erlfdb_tenant_management.list_tenants/4","doc":"","ref":"erlfdb_tenant_management.html#list_tenants/4"},{"type":"function","title":"erlfdb_tenant_management.transactional/2","doc":"","ref":"erlfdb_tenant_management.html#transactional/2"},{"type":"module","title":"erlfdb_tuple","doc":"[Tuple Layer](https://github.com/apple/foundationdb/blob/main/design/tuple.md)\n\n[Data Modeling with Tuples](https://apple.github.io/foundationdb/data-modeling.html#data-modeling-tuples)","ref":"erlfdb_tuple.html"},{"type":"function","title":"erlfdb_tuple.compare/2","doc":"Compares 2 Erlang tuples with the ordering defined by the tuple layer.","ref":"erlfdb_tuple.html#compare/2"},{"type":"function","title":"Return - erlfdb_tuple.compare/2","doc":"- `0` if both tuples compare as equal.\n- `-1` if the left tuple is less than the right tuple.\n- `1` if the left tuple is greater than the right tuple.","ref":"erlfdb_tuple.html#compare/2-return"},{"type":"function","title":"Examples - erlfdb_tuple.compare/2","doc":"The ordering isn't equivalent to the Erlang term ordering:\n\n```erlang\n1> lists:sort(fun(A, B) -> erlfdb_tuple:compare(A, B) =< 0 end, [{1}, {null}]).\n[{null},{1}]\n2> lists:sort([{1}, {null}]).\n[{1},{null}]\n```","ref":"erlfdb_tuple.html#compare/2-examples"},{"type":"function","title":"erlfdb_tuple.pack/1","doc":"Encodes the tuple into a binary.","ref":"erlfdb_tuple.html#pack/1"},{"type":"function","title":"Supported Erlang types - erlfdb_tuple.pack/1","doc":"- `t:binary/0`: Encodes as binary blob.\n- `t:integer/0`: Encodes as variable-length integer.\n- `t:float/0`: Encodes as 64-bit floating point (double).\n- `t:tuple/0`: Encodes as nested tuple.","ref":"erlfdb_tuple.html#pack/1-supported-erlang-types"},{"type":"function","title":"Special types - erlfdb_tuple.pack/1","doc":"- `null`: Encodes as NULL char (`16#0`)  instead of as an atom.\n- `t:boolean/0`: Encodes as a boolean instead of as an atom.\n- `{utf8, Bin :: binary()}`: Encodes `Bin` as UTF-8 string.\n- `{float, float()}`: Advanced floating point encoding. See `m:erlfdb_float`.\n- `{float, _, float()}`: Advanced floating point encoding. See `m:erlfdb_float`.\n- `{double, _, float()}`: Advanced floating point encoding. See `m:erlfdb_float`.\n- `{uuid, binary()}`: Encodes a binary of size == 16 as a UUID type.\n- `{id64, integer()}`: Encodes a fixed-length 64-bit integer.\n- `{versionstamp, Id :: integer(), Batch :: integer()}`: Versionstamp encoding. For unset versionstamps use `pack_vs/1`.\n- `{versionstamp, Id :: integer(), Batch :: integer(), Tx}`: Versionstamp encoding. For unset versionstamps use `pack_vs/1`.","ref":"erlfdb_tuple.html#pack/1-special-types"},{"type":"function","title":"Unsupported Erlang types - erlfdb_tuple.pack/1","doc":"- `t:atom/0`\n- `t:list/0`\n- `t:reference/0`\n- `t:pid/0`\n- `t:function/0`\n- `t:map/0`","ref":"erlfdb_tuple.html#pack/1-unsupported-erlang-types"},{"type":"function","title":"Examples - erlfdb_tuple.pack/1","doc":"```erlang\n1> erlfdb_tuple:pack({}).\n<<>>\n```\n\n```erlang\n1> erlfdb_tuple:pack({<<\"hello\">>, 42}).\n<<1,104,101,108,108,111,0,21,42>>\n```","ref":"erlfdb_tuple.html#pack/1-examples"},{"type":"function","title":"erlfdb_tuple.pack/2","doc":"Encodes the prefix and tuple into a binary.","ref":"erlfdb_tuple.html#pack/2"},{"type":"function","title":"Examples - erlfdb_tuple.pack/2","doc":"```erlang\n1> erlfdb_tuple:pack({}, <<\"foo\">>).\n<<\"foo\">>\n```\n\n```erlang\n1> erlfdb_tuple:pack({<<\"foo\">>}, <<>>).\n<<1,102,111,111,0>>\n```\n\n```erlang\n1> erlfdb_tuple:pack({<<\"hello\">>, 42}, <<\"foo\">>).\n<<102,111,111,1,104,101,108,108,111,0,21,42>>\n```","ref":"erlfdb_tuple.html#pack/2-examples"},{"type":"function","title":"erlfdb_tuple.pack_vs/1","doc":"With versionstamps, encodes the tuple into a binary.","ref":"erlfdb_tuple.html#pack_vs/1"},{"type":"function","title":"Special types - erlfdb_tuple.pack_vs/1","doc":"- `{versionstamp, Id, Batch}`: Versionstamp encoding\n- `{versionstamp, Id, Batch, Tx}`: Versionstamp encoding","ref":"erlfdb_tuple.html#pack_vs/1-special-types"},{"type":"function","title":"erlfdb_tuple.pack_vs/2","doc":"With versionstamps, encodes the prefix and tuple into a binary.","ref":"erlfdb_tuple.html#pack_vs/2"},{"type":"function","title":"erlfdb_tuple.range/1","doc":"Returns a `{StartKey, EndKey}` pair of binaries that includes all possible sub-tuples.","ref":"erlfdb_tuple.html#range/1"},{"type":"function","title":"erlfdb_tuple.range/2","doc":"With prefix, returns a `{StartKey, EndKey}` pair of binaries that includes all possible sub-tuples.","ref":"erlfdb_tuple.html#range/2"},{"type":"function","title":"erlfdb_tuple.unpack/1","doc":"Decodes the binary into a tuple.","ref":"erlfdb_tuple.html#unpack/1"},{"type":"function","title":"Examples - erlfdb_tuple.unpack/1","doc":"```erlang\n1> erlfdb_tuple:unpack(erlfdb_tuple:pack({})).\n{}\n```\n\n```erlang\n1> erlfdb_tuple:unpack(erlfdb_tuple:pack({<<\"hello\">>, 42})).\n{<<\"hello\">>,42}\n```","ref":"erlfdb_tuple.html#unpack/1-examples"},{"type":"function","title":"erlfdb_tuple.unpack/2","doc":"Decodes the binary into a prefix and tuple.","ref":"erlfdb_tuple.html#unpack/2"},{"type":"function","title":"Examples - erlfdb_tuple.unpack/2","doc":"```erlang\n1> erlfdb_tuple:unpack(erlfdb_tuple:pack({}, <<\"foo\">>), <<\"foo\">>).\n{}\n```\n\n```erlang\n1> erlfdb_tuple:unpack(erlfdb_tuple:pack({<<\"hello\">>, 42}, <<\"foo\">>), <<\"foo\">>).\n{<<\"hello\">>,42}\n```","ref":"erlfdb_tuple.html#unpack/2-examples"},{"type":"extras","title":"An Erlang Binding to FoundationDB","doc":"# An Erlang Binding to FoundationDB\n\n[![CI](https://github.com/foundationdb-beam/erlfdb/actions/workflows/ci.yml/badge.svg)](https://github.com/foundationdb-beam/erlfdb/actions/workflows/ci.yml)\n\nAn Erlang library that wraps the [FoundationDB C API](https://apple.github.io/foundationdb/api-c.html) with a NIF.\n\nWe also provide a conforming implementation of the [Tuple] and [Directory] layers.\n\n[Tuple]: https://github.com/apple/foundationdb/blob/master/design/tuple.md\n[Directory]: https://apple.github.io/foundationdb/developer-guide.html#directories\n\nThis project originated as [apache/couchdb-erlfdb](https://github.com/apache/couchdb-erlfdb). This fork\nwas created in early 2024 to continue development.","ref":"readme.html"},{"type":"extras","title":"Dependencies - An Erlang Binding to FoundationDB","doc":"Refer to FoundationDB's [Getting Started on Linux] or [Getting Started on macOS].\n\n[Getting Started on Linux]: https://apple.github.io/foundationdb/getting-started-linux.html\n[Getting Started on macOS]: https://apple.github.io/foundationdb/getting-started-mac.html\n\nInstall the foundationdb-clients package to run erlfdb as a client to an existing\nFoundationDB database.\n\nInstall the foundationdb-server package for running a test fdbserver on your local device.\nThis package is required to run the unit tests.","ref":"readme.html#dependencies"},{"type":"extras","title":"Usage - An Erlang Binding to FoundationDB","doc":"<!-- tabs-open -->","ref":"readme.html#usage"},{"type":"extras","title":"Erlang's rebar3 - An Erlang Binding to FoundationDB","doc":"Add erlfdb as a dependency in your Erlang project's rebar.config:\n\n```erlang\n% rebar.config\n{deps, [\n    {erlfdb, \"0.3.0\"}\n]}.\n```","ref":"readme.html#erlang-s-rebar3"},{"type":"extras","title":"Elixir's Mix - An Erlang Binding to FoundationDB","doc":"Add erlfdb as a dependency in your Elixir project's mix.exs:\n\n```elixir\n# mix.exs\ndefp deps do\n  [\n    {:erlfdb, \"~> 0.3\"}\n  ]\nend\n```\n\n<!-- tabs-close -->","ref":"readme.html#elixir-s-mix"},{"type":"extras","title":"Example - An Erlang Binding to FoundationDB","doc":"A simple example showing how to open a database and read and write keys.\n\nSee the [erlfdb Documentation](https://hexdocs.pm/erlfdb/erlfdb.html) for more.\n\n<!-- tabs-open -->","ref":"readme.html#example"},{"type":"extras","title":"Erlang - An Erlang Binding to FoundationDB","doc":"```erlang\n1> Db = erlfdb:open(<<\"/usr/local/etc/foundationdb/fdb.cluster\">>).\n{erlfdb_database,#Ref<0.2859661758.3941466120.85406>}\n2> ok = erlfdb:set(Db, <<\"foo\">>, <<\"bar\">>).\nok\n3> erlfdb:get(Db, <<\"foo\">>).\n<<\"bar\">>\n4> erlfdb:get(Db, <<\"bar\">>).\nnot_found\n```","ref":"readme.html#erlang"},{"type":"extras","title":"Elixir - An Erlang Binding to FoundationDB","doc":"```elixir\niex> db = :erlfdb.open(\"/usr/local/etc/foundationdb/fdb.cluster\")\n{:erlfdb_database, #Reference<0.2859661758.3941466120.85406>}\niex> :ok = :erlfdb.set(db, \"foo\", \"bar\")\n:ok\niex> :erlfdb.get(db, \"foo\")\n\"bar\"\niex> :erlfdb.get(db, \"bar\")\n:not_found\n```\n\n<!-- tabs-close -->","ref":"readme.html#elixir"},{"type":"extras","title":"Binding Tester - An Erlang Binding to FoundationDB","doc":"FoundationDB has a custom binding tester that can be used to test whether\nchanges have broken compatibility. The GitHub Action runs the Binding Tester\nagainst the most recent supported version of FoundationDB.","ref":"readme.html#binding-tester"},{"type":"extras","title":"Developing erlfdb - An Erlang Binding to FoundationDB","doc":"","ref":"readme.html#developing-erlfdb"},{"type":"extras","title":"Building - An Erlang Binding to FoundationDB","doc":"$ rebar3 compile","ref":"readme.html#building"},{"type":"extras","title":"Testing - An Erlang Binding to FoundationDB","doc":"# rebar3 eunit","ref":"readme.html#testing"},{"type":"extras","title":"Bypassing dependency checks - An Erlang Binding to FoundationDB","doc":"When you execute a rebar command, erlfdb attempts to detect the version of the fdbcli\ninstalled on your system. If it cannot be detected, the rebar command is aborted.\n\nTo disable the abort, use `ERLFDB_ASSERT_FDBCLI=0`. For example, you can safely use\nthis for the `fmt` command, which does not do a compile action.\n\n```bash\nERLFDB_ASSERT_FDBCLI=0 rebar3 fmt\n```","ref":"readme.html#bypassing-dependency-checks"},{"type":"extras","title":"LICENSE","doc":"Apache License\n                          Version 2.0, January 2004\n                       http://www.apache.org/licenses/\n\n  TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n  1. Definitions.\n\n     \"License\" shall mean the terms and conditions for use, reproduction,\n     and distribution as defined by Sections 1 through 9 of this document.\n\n     \"Licensor\" shall mean the copyright owner or entity authorized by\n     the copyright owner that is granting the License.\n\n     \"Legal Entity\" shall mean the union of the acting entity and all\n     other entities that control, are controlled by, or are under common\n     control with that entity. For the purposes of this definition,\n     \"control\" means (i) the power, direct or indirect, to cause the\n     direction or management of such entity, whether by contract or\n     otherwise, or (ii) ownership of fifty percent (50%) or more of the\n     outstanding shares, or (iii) beneficial ownership of such entity.\n\n     \"You\" (or \"Your\") shall mean an individual or Legal Entity\n     exercising permissions granted by this License.\n\n     \"Source\" form shall mean the preferred form for making modifications,\n     including but not limited to software source code, documentation\n     source, and configuration files.\n\n     \"Object\" form shall mean any form resulting from mechanical\n     transformation or translation of a Source form, including but\n     not limited to compiled object code, generated documentation,\n     and conversions to other media types.\n\n     \"Work\" shall mean the work of authorship, whether in Source or\n     Object form, made available under the License, as indicated by a\n     copyright notice that is included in or attached to the work\n     (an example is provided in the Appendix below).\n\n     \"Derivative Works\" shall mean any work, whether in Source or Object\n     form, that is based on (or derived from) the Work and for which the\n     editorial revisions, annotations, elaborations, or other modifications\n     represent, as a whole, an original work of authorship. For the purposes\n     of this License, Derivative Works shall not include works that remain\n     separable from, or merely link (or bind by name) to the interfaces of,\n     the Work and Derivative Works thereof.\n\n     \"Contribution\" shall mean any work of authorship, including\n     the original version of the Work and any modifications or additions\n     to that Work or Derivative Works thereof, that is intentionally\n     submitted to Licensor for inclusion in the Work by the copyright owner\n     or by an individual or Legal Entity authorized to submit on behalf of\n     the copyright owner. For the purposes of this definition, \"submitted\"\n     means any form of electronic, verbal, or written communication sent\n     to the Licensor or its representatives, including but not limited to\n     communication on electronic mailing lists, source code control systems,\n     and issue tracking systems that are managed by, or on behalf of, the\n     Licensor for the purpose of discussing and improving the Work, but\n     excluding communication that is conspicuously marked or otherwise\n     designated in writing by the copyright owner as \"Not a Contribution.\"\n\n     \"Contributor\" shall mean Licensor and any individual or Legal Entity\n     on behalf of whom a Contribution has been received by Licensor and\n     subsequently incorporated within the Work.\n\n  2. Grant of Copyright License. Subject to the terms and conditions of\n     this License, each Contributor hereby grants to You a perpetual,\n     worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n     copyright license to reproduce, prepare Derivative Works of,\n     publicly display, publicly perform, sublicense, and distribute the\n     Work and such Derivative Works in Source or Object form.\n\n  3. Grant of Patent License. Subject to the terms and conditions of\n     this License, each Contributor hereby grants to You a perpetual,\n     worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n     (except as stated in this section) patent license to make, have made,\n     use, offer to sell, sell, import, and otherwise transfer the Work,\n     where such license applies only to those patent claims licensable\n     by such Contributor that are necessarily infringed by their\n     Contribution(s) alone or by combination of their Contribution(s)\n     with the Work to which such Contribution(s) was submitted. If You\n     institute patent litigation against any entity (including a\n     cross-claim or counterclaim in a lawsuit) alleging that the Work\n     or a Contribution incorporated within the Work constitutes direct\n     or contributory patent infringement, then any patent licenses\n     granted to You under this License for that Work shall terminate\n     as of the date such litigation is filed.\n\n  4. Redistribution. You may reproduce and distribute copies of the\n     Work or Derivative Works thereof in any medium, with or without\n     modifications, and in Source or Object form, provided that You\n     meet the following conditions:\n\n     (a) You must give any other recipients of the Work or\n         Derivative Works a copy of this License; and\n\n     (b) You must cause any modified files to carry prominent notices\n         stating that You changed the files; and\n\n     (c) You must retain, in the Source form of any Derivative Works\n         that You distribute, all copyright, patent, trademark, and\n         attribution notices from the Source form of the Work,\n         excluding those notices that do not pertain to any part of\n         the Derivative Works; and\n\n     (d) If the Work includes a \"NOTICE\" text file as part of its\n         distribution, then any Derivative Works that You distribute must\n         include a readable copy of the attribution notices contained\n         within such NOTICE file, excluding those notices that do not\n         pertain to any part of the Derivative Works, in at least one\n         of the following places: within a NOTICE text file distributed\n         as part of the Derivative Works; within the Source form or\n         documentation, if provided along with the Derivative Works; or,\n         within a display generated by the Derivative Works, if and\n         wherever such third-party notices normally appear. The contents\n         of the NOTICE file are for informational purposes only and\n         do not modify the License. You may add Your own attribution\n         notices within Derivative Works that You distribute, alongside\n         or as an addendum to the NOTICE text from the Work, provided\n         that such additional attribution notices cannot be construed\n         as modifying the License.\n\n     You may add Your own copyright statement to Your modifications and\n     may provide additional or different license terms and conditions\n     for use, reproduction, or distribution of Your modifications, or\n     for any such Derivative Works as a whole, provided Your use,\n     reproduction, and distribution of the Work otherwise complies with\n     the conditions stated in this License.\n\n  5. Submission of Contributions. Unless You explicitly state otherwise,\n     any Contribution intentionally submitted for inclusion in the Work\n     by You to the Licensor shall be under the terms and conditions of\n     this License, without any additional terms or conditions.\n     Notwithstanding the above, nothing herein shall supersede or modify\n     the terms of any separate license agreement you may have executed\n     with Licensor regarding such Contributions.\n\n  6. Trademarks. This License does not grant permission to use the trade\n     names, trademarks, service marks, or product names of the Licensor,\n     except as required for reasonable and customary use in describing the\n     origin of the Work and reproducing the content of the NOTICE file.\n\n  7. Disclaimer of Warranty. Unless required by applicable law or\n     agreed to in writing, Licensor provides the Work (and each\n     Contributor provides its Contributions) on an \"AS IS\" BASIS,\n     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n     implied, including, without limitation, any warranties or conditions\n     of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n     PARTICULAR PURPOSE. You are solely responsible for determining the\n     appropriateness of using or redistributing the Work and assume any\n     risks associated with Your exercise of permissions under this License.\n\n  8. Limitation of Liability. In no event and under no legal theory,\n     whether in tort (including negligence), contract, or otherwise,\n     unless required by applicable law (such as deliberate and grossly\n     negligent acts) or agreed to in writing, shall any Contributor be\n     liable to You for damages, including any direct, indirect, special,\n     incidental, or consequential damages of any character arising as a\n     result of this License or out of the use or inability to use the\n     Work (including but not limited to damages for loss of goodwill,\n     work stoppage, computer failure or malfunction, or any and all\n     other commercial damages or losses), even if such Contributor\n     has been advised of the possibility of such damages.\n\n  9. Accepting Warranty or Additional Liability. While redistributing\n     the Work or Derivative Works thereof, You may choose to offer,\n     and charge a fee for, acceptance of support, warranty, indemnity,\n     or other liability obligations and/or rights consistent with this\n     License. However, in accepting such obligations, You may act only\n     on Your own behalf and on Your sole responsibility, not on behalf\n     of any other Contributor, and only if You agree to indemnify,\n     defend, and hold each Contributor harmless for any liability\n     incurred by, or claims asserted against, such Contributor by reason\n     of your accepting any such warranty or additional liability.\n\n  END OF TERMS AND CONDITIONS\n\n  APPENDIX: How to apply the Apache License to your work.\n\n     To apply the Apache License to your work, attach the following\n     boilerplate notice, with the fields enclosed by brackets \"[]\"\n     replaced with your own identifying information. (Don't include\n     the brackets!)  The text should be enclosed in the appropriate\n     comment syntax for the file format. We also recommend that a\n     file or class name and description of purpose be included on the\n     same \"printed page\" as the copyright notice for easier\n     identification within third-party archives.\n\n  Copyright [yyyy] [name of copyright owner]\n\n  Licensed under the Apache License, Version 2.0 (the \"License\");\n  you may not use this file except in compliance with the License.\n  You may obtain a copy of the License at\n\n      http://www.apache.org/licenses/LICENSE-2.0\n\n  Unless required by applicable law or agreed to in writing, software\n  distributed under the License is distributed on an \"AS IS\" BASIS,\n  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n  See the License for the specific language governing permissions and\n  limitations under the License.","ref":"license.html"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v0.3 (2025-04-03) - Changelog","doc":"","ref":"changelog.html#v0-3-2025-04-03"},{"type":"extras","title":"Enhancements - Changelog","doc":"* (#46) Add support for `client_threads_per_version` config. See [FDB Client Threads](thread-design.html) for details.","ref":"changelog.html#enhancements"},{"type":"extras","title":"New contributors - Changelog","doc":"Thank you to the following new contributors! :)\n\n  * weaversam8","ref":"changelog.html#new-contributors"},{"type":"extras","title":"v0.2.4 (2025-03-23) - Changelog","doc":"","ref":"changelog.html#v0-2-4-2025-03-23"},{"type":"extras","title":"Bug fixes - Changelog","doc":"* (#43) For OTP >= 26, erlfdb NIF will no longer segfault on VM halt.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.2.3 (2025-03-15) - Changelog","doc":"","ref":"changelog.html#v0-2-3-2025-03-15"},{"type":"extras","title":"Bug fixes - Changelog","doc":"* (#36) Starting in v0.2.2, the ready message from a get_versionstamp future included\n    the ref from the transaction, putting it at risk of being flushed in the after the\n    transaction completes. Now, the ready message behaves much like the watch, where it\n    is not tied to the transaction ref.\n  * Sandboxes can now be specified with a subdirectory, allowing concurrent\n    creation (`m:erlfdb_sandbox`)","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Enhancements - Changelog","doc":"* [Livebook | KvQueue](kv_queue.html): An interactive notebook that\n demonstrates the creation of a durable queue in FoundationDB.","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.2.2 (2024-01-16) - Changelog","doc":"","ref":"changelog.html#v0-2-2-2024-01-16"},{"type":"extras","title":"Bug fixes - Changelog","doc":"* (#31) Previously, erlfdb could leak `{reference(), ready}` messages to the caller if\n    the transaction UserFun was executed more than once. We will now flush such messages before\n    `transactional/2` returns control to the caller. Watches are unaffected.\n  * (#32) Fixed documentation for versionstamps in `m:erlfdb_tuple`.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Enhancements - Changelog","doc":"* `m:erlfdb_sandbox`: Creates and starts a database on your local filesystem that is to be\n    used as a sandbox. That is, it can be useful for development tasks like tutorials and unit tests.\n    It should not be used in a production setting.","ref":"changelog.html#enhancements"},{"type":"extras","title":"New contributors - Changelog","doc":"Thank you to the following new contributors! :)\n\n* drowzy","ref":"changelog.html#new-contributors"},{"type":"extras","title":"v0.2.1 (2024-11-20) - Changelog","doc":"","ref":"changelog.html#v0-2-1-2024-11-20"},{"type":"extras","title":"Testing - Changelog","doc":"* The FDB Bindings Tester now runs under GitHub Actions\n  * Added macOS GitHub Action","ref":"changelog.html#testing"},{"type":"extras","title":"v0.2.0 (2024-09-21) - Changelog","doc":"","ref":"changelog.html#v0-2-0-2024-09-21"},{"type":"extras","title":"Bug fixes - Changelog","doc":"* Several type specs in `erlfdb` were corrected.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Enhancements - Changelog","doc":"* `erlfdb:wait_for_all_interleaving/2`: Given a list of fold_future() or future(), calling this function will wait\n    on the futures at once, and then continue to issue any remaining get_range or get_mapped_range until\n    the result set is exhausted. This allows fdbserver to process multiple get_ranges at the same time,\n    in a pipelined fashion.\n  * `erlfdb:get_range_split_points/4`: Companion to `wait_for_all_interleaving`, this is an fdbserver-supported function\n    that will split a given key range into a partitioning set of ranges for which the key-values for each section are\n    approximately equal to the provided `chunk_size` option. There are limitations to this, namely that a hard\n    maximum of 100 shards can be traversed. The default `chunk_size` is 10000000 (in Bytes).\n  * `erlfdb:get_range*`: The default behavior of get_range is now more explicit in the type specs and with the `wait`\n    option, with defaults to `true`. A value of `false` will return a fold_future(), and `interleaving` is\n    an experimental feature that will use both `get_range_split_points` and `wait_for_all_interleaving` to retrieve the range.","ref":"changelog.html#enhancements"},{"type":"extras","title":"FDB Client Threads","doc":"# FDB Client Threads\n\nThe default configuration of erlfdb includes the creation and management of a single OS thread that is responsible for all database work created by processes in the Erlang VM. As your workload increases in size, you will need to configure the erlfdb client for horizontal scaling. Otherwise, a sufficiently sized system with a sufficiently sized workload can saturate the single client thread with work, which will lead to high latency on individual requests, and a ceiling on throughput.\n\nThis page will guide you through the configuration necessary to scale the FDB client to several worker threads on a single BEAM VM.","ref":"thread-design.html"},{"type":"extras","title":"Network options configuration - FDB Client Threads","doc":"The configuration will be done via the erlfdb `network_options` environment variable. In erlfdb >= 0.3, defaults are chosen that are meant to ease the configuration burden. The options are described in detail below, but the main takeaway is the following:\n\n> #### Tip {: .tip}\n>\n> In order to double your client's maximum throughput, set\n> `client_threads_per_version` to 2.\n\n```erlang\n[\n    {erlfdb, [\n        {network_options, [\n            {client_threads_per_version, 2}\n        ]}\n    ]}\n].\n```\n\nPlease read on for details about the scaling implementation.","ref":"thread-design.html#network-options-configuration"},{"type":"extras","title":"Version 0.3 and above - FDB Client Threads","doc":"In versions 0.3 and above, the env var `network_options` defaults to\n\n```erlang\n[\n    {erlfdb, [\n        {network_options, [\n            % `ignore_external_client_failures`:\n            % Optional: Allows your client to prepare fdbserver version\n            % upgrades. When a future client library fails to load, it's not a\n            % failure condition, because we know it will be valid at some\n            % point in the future.\n            {ignore_external_client_failures, true},\n\n            % `callbacks_on_external_threads`:\n            % Allows libfdb_c to apply future callbacks on external threads,\n            % which increases the throughput of future resolution. erlfdb\n            % simply sends the result to the required Erlang process, so the\n            % default behavior of executing on the main thread is not\n            % desirable\n            {callbacks_on_external_threads, true},\n\n            % `external_client_directory`:\n            % Points libfdb_c to the directory containing the dynamic\n            % library(ies) to be loaded. libfdb_c will copy each .so N times\n            % to temporary files on the filesystem so that they can be\n            % individually loaded. Normally the value is a binary string of\n            % the directory on the filesystem containing the dynamic\n            % library(ies).\n            %\n            % You may also provide an `{M, F, A}` tuple. If you do, the function\n            % is executed at the time the `erlfdb_nif` module is loaded. The\n            % return must be a binary string. This value is used as the\n            % permanent value for `external_client_directory` for the lifetime\n            % of the VM.\n            {external_client_directory, {erlfdb_network_options, compile_time_external_client_directory, []}},\n\n            % `client_threads_per_version`:\n            % Number of threads to create per dynamic library. This is the\n            % value to tweak in order to scale the number of client threads\n            % horizontally. This value is used as the permanent value\n            % for the lifetime of the BEAM VM.\n            %\n            % When the value is 1, only the local client is used. When the\n            % value is N > 1, then N external client threads are used.\n            %\n            % You may also provide an `{M, F, A}` tuple. If you do, the\n            % function is executed at the time the `erlfdb_nif` module is\n            % loaded. The return must be a positive integer.\n            %\n            % Note: For each thread, libfdb_c will create a tcp\n            % connection for each coordinator in the cluster file.\n            {client_threads_per_version, 1}\n        ]}\n    ]}\n].\n```\n\nWhen the erlfdb NIF is loaded, any network_options defined by your config are merged into the defaults, so that yours always take precedence. The final set of resolved network_options is stored in env var `network_options_resolved`.\n\n```erlang\napplication:get_env(erlfdb, network_options_resolved).\n```","ref":"thread-design.html#version-0-3-and-above"},{"type":"extras","title":"How to choose the correct value for `client_threads_per_version` - FDB Client Threads","doc":"The ideal approach is to choose an integer value that is sufficient for your workload, and no more. It's not recommended to choose a value that is larger than the number of online schedulers.\n\nInstead of choosing a specific value, you may wish for erlfdb to scale dynamically along with your scaling of the Erlang VM itself. Consider the following MFA-tuple choices.\n\n- `{erlang, system_info, [schedulers_online]}`\n- `{erlang, system_info, [dirty_io_schedulers]}`\n\n> #### Info {: .info}\n> These suggestions are made simply as a convenience for automatic scaling, and none of these choices will associate erlfdb external client threads with Erlang's schdeulers or dirty_io_schedulers. External client threads will always be created **in addition to** and **not replacements for** any of the Erlang VM's OS threads.","ref":"thread-design.html#how-to-choose-the-correct-value-for-client_threads_per_version"},{"type":"extras","title":"Making use of External Client Threads - FDB Client Threads","doc":"","ref":"thread-design.html#making-use-of-external-client-threads"},{"type":"extras","title":"Distributing your workfload - FDB Client Threads","doc":"So you've configured the client to create multiple external client threads. Now, you need to make sure your application distributes the workload among these threads.\n\nThe function `erlfdb:open/2` provides your application with a convenient way to distribute your workload. Whenever you need a database object from which to create a transaction, call `open` to retrieve one very quickly (via persistent_term storage). With the default work distribution strategy (`scheduler_id`), erlfdb will create and keep track of N database objects where N is the number of schedulers on your system. Each of these database objects will automatically be assigned to one of the external client threads in libfdb_c.\n\n> #### Info {: .info}\n> This strategy does not claim that a particular item of work will remain on an Erlang scheduler -- only that the scheduler_id itself is helpful in distributing work in a roughly even fashion.\n\nIt may be beneficial to create all database objects during startup. The function `erlfdb:open_all/2` is provided for this purpose.","ref":"thread-design.html#distributing-your-workfload"},{"type":"extras","title":"Thread lifecycle - FDB Client Threads","doc":"When `client_threads_per_version` == 1,\n\n1. `enif_thread_create` is called exactly once. This is the 'local client network thread' a.k.a. the libfdb_c Main Thread. This thread is given the name `fdb:network_thread`.\n2. Shutdown: The erlfdb NIF uses enif callbacks to ensure the event loop is stopped and the thread is joined before the VM is terminated.\n\nWhen `client_threads_per_version` > 1, the behavior described above is true; additionally:\n\n1. libfdb_c creates N copies of the dynamic library into temp files on the filesystem. Each copy will house 1 of the external threads.\n2. N threads are created by libfdb_c. These threads are *not* created with `enif_thread_create`, because the creation is contained in logic internal to libfdb_c. Each of these threads is given the name `fdb- - `. If the thread name is longer than 15 chars, it's instead given the name `fdb- `. If this is longer than 15 chars, it's given the name `fdb-external`. On a Linux system, these threads are visible with `top -H -p $beam_pid`.\n3. Each database object (via `erlfdb:create_database/1`) is linked to a client thread at the time of creation. The threads are distributed in a round-robin fashion. Therefore, to make use of N client threads, you must have N database objects.\n4. Shutdown: Each external thread is waited upon immediately after the local client network event loop returns. Thus, you may consider the external client threads as \"children\" of the local client network thread created by `enif_thread_create`. This relationship is necessary and sufficient for the Erlang VM and its operator to maintain control over the OS threads on the system.","ref":"thread-design.html#thread-lifecycle"},{"type":"extras","title":"Tracing - FDB Client Threads","doc":"To enable client tracing (C API Option `FDB_NET_OPTION_TRACE_ENABLE`), set the following:\n\n```erlang\n[\n    {erlfdb, [\n        {network_options, [\n            % `trace_enable`:\n            % A directory on the filesystem where trace files will be written.\n            {trace_enable, <<\"/path/to/existing/directory\">>},\n\n            % `trace_format`:\n            % The format of the trace files. Supported formats are `<<\"xml\">>` (the default) and `<<\"json\">>`.\n            {trace_format, <<\"xml\">>}\n        ]}\n    ]}\n].\n```","ref":"thread-design.html#tracing"},{"type":"extras","title":"Past Versions 0.0 - 0.2 - FDB Client Threads","doc":"In versions 0.0.x - 0.2.x, the env var `network_options` defaults to `[]`. The behavior of the erlfdb NIF with respect to thread creation is equivalent to having `client_threads_per_version` == 1, as decribed above.\n\nIn these erlfdb versions, the `client_threads_per_version` env var is not supported, so horizontal scaling of a single client is not possible. Instead, consider starting multiple Erlang VMs in order to distribute your workload, or upgrade to erlfdb >= 0.3.","ref":"thread-design.html#past-versions-0-0-0-2"},{"type":"extras","title":"References - FDB Client Threads","doc":"A short list of suggested reading in the FoundationDB source that will help the reader understand the details of the client threading model.\n\n1. `fdbclient/MultiVersionTransaction.actor.cpp`: implements the management of the local and external clients.\n2. `bindings/c/fdb_c.cpp`: defines MultiVersionApi as the default implementation\n3. `fdbclient/NativeAPI.actor.cpp`: implements the client (local or external). Specifically, maintains the reference to the thread.\n4. `flow/Net2.actor.cpp`: implements the event loop","ref":"thread-design.html#references"},{"type":"extras","title":"KvQueue - A distributed durable queue with erlfdb","doc":"<!-- livebook:{\"default_language\":\"erlang\"} -->\n\n# KvQueue - A distributed durable queue with erlfdb\n\n```elixir\nMix.install([\n  {:erlfdb, \"~> 0.3\"}\n])\n```","ref":"kv_queue.html"},{"type":"extras","title":"Introduction - KvQueue - A distributed durable queue with erlfdb","doc":"The FoundationDB [Layer Concept](https://apple.github.io/foundationdb/layer-concept.html) means that our query execution is detached from our storage servers, allowing a database operator to dynamically scale compute nodes according to current query workloads. However, there is also an advantage for application developers: when combined with transactions, a Layer allows us to create powerful data structures as building blocks for an application.\n\nIn this tutorial, we'll create a Layer that is a performant durable queue. Then, we'll consume items from this queue using a `gen_server`.\n\nOur queue will prioritize throughput of the producer. This is a common design choice for a durable queue because it ensures our producer can quickly move on to other operations in our application, without costly locks or conflicts to slow it down.\n\nWith transactional isolation guarantees from FDB, we can create as many parallel and distributed producers and consumers as we like; a given message will always be consumed exactly once.","ref":"kv_queue.html#introduction"},{"type":"extras","title":"Start a sandbox - KvQueue - A distributed durable queue with erlfdb","doc":"First, we'll create a sandbox database that we can freely write to and delete from. `erlfdb_sandbox` starts a single fdbserver process and writes data to a temporary directory. It may take a few seconds the first time you execute it.\n\n```erlang\nDb = erlfdb_sandbox:open().\n```","ref":"kv_queue.html#start-a-sandbox"},{"type":"extras","title":"Defining the Queue - KvQueue - A distributed durable queue with erlfdb","doc":"We'll define a standard FIFO queue with push and pop operations. Our main challenge is laying out the queue onto FDB key-value pairs. It's up to us to design a keyspace that will fit our design specifications. So, let's discuss it now.","ref":"kv_queue.html#defining-the-queue"},{"type":"extras","title":"Key-value binary format - KvQueue - A distributed durable queue with erlfdb","doc":"Our keyspace will use the FDB [Tuple Layer](https://apple.github.io/foundationdb/data-modeling.html#tuples) for constructing the key format. We'll document our keys and values with this simple notation:\n\n```\n{ } =>  \n```","ref":"kv_queue.html#key-value-binary-format"},{"type":"extras","title":"Queue name - KvQueue - A distributed durable queue with erlfdb","doc":"We're ready to talk keys! First, we'll hard-code the name of our queue to `<<\"q\">>`. This makes our queue a singleton in the database, which keeps our code concise for this tutorial. The data structures you create should allow a developer to specify certain details like the name to avoid key conflicts.\n\nAll our key-values will be of the form:\n\n```\n{q, ...} =>  \n```","ref":"kv_queue.html#queue-name"},{"type":"extras","title":"Queue length - KvQueue - A distributed durable queue with erlfdb","doc":"Next, in order to keep track of the length of the queue, we'll keep track of counter values called `<<\"npush\">>` and `<<\"npop\">>`, which are the total number of pushes and pops executed,\nrespectively. Splitting the counters is an important part of how the consumer works, which we'll get to later.\n\n```\n{q, npush} =>  \n{q, npop} =>  \n```\n\nAt any given time, the length of the queue is equal to `  -  `. We'll have to make sure these counters are safely managed so that there are no race conditions. (We don't want negative values!)","ref":"kv_queue.html#queue-length"},{"type":"extras","title":"Queue items - KvQueue - A distributed durable queue with erlfdb","doc":"Finally, keys for items in the queue include `<<\"val\">>` in the key:\n\n```\n{q, val,  } =>  \n```\n\nwhere ` ` is a placeholder for a versionstamp, and ` ` is the content of item stored in the queue.\n\nA [Versionstamp](https://apple.github.io/foundationdb/data-modeling.html#versionstamps) is a unique monotinically increasing integer generated by FDB at commit-time. We have to use special client-side functions that operate on \"incomplete versionstamps\" because the client cannot know what the value of the versionstamp is before the commit.\n\nUsing a versionstamp for pushing items onto the queue ensures that there is no key contention for parallel pushes, which greatly increases producer throughput.","ref":"kv_queue.html#queue-items"},{"type":"extras","title":"Code time! - KvQueue - A distributed durable queue with erlfdb","doc":"Once you have a good grasp of the keyspace, writing the code becomes pretty straightforward!\n\n```erlang\n-module(kv_queue).\n-export([\n         delete/1,\n         push/2,\n         consume_k/2,\n         pop/1,\n         pop_k/2,\n         len/1,\n         queue/1,\n         watch/1\n        ]).\n\n-define(NAME, <<\"q\">>).\n-define(VAL, <<\"val\">>).\n-define(NPOP, <<\"npop\">>).\n-define(NPUSH, <<\"npush\">>).\n\n% This is known as an \"incomplete versionstamp\"\n-define(VS(Tx), {\n      versionstamp,\n      16#ffffffffffffffff,\n      16#ffff,\n      erlfdb:get_next_tx_id(Tx)\n    }).\n\ndelete(Tx) ->\n  {S, E} = erlfdb_tuple:range({?NAME}),\n  erlfdb:clear_range(Tx, S, E).\n\npush(Tx, Val) ->\n  Key = erlfdb_tuple:pack_vs({?NAME, ?VAL, ?VS(Tx)}),\n  erlfdb:set_versionstamped_key(Tx, Key, Val),\n  erlfdb:add(Tx, erlfdb_tuple:pack({?NAME, ?NPUSH}), 1).\n\npop(Tx) ->\n  case pop_k(Tx, 1) of\n    {_, [Val]} -> {ok, Val};\n    {_, []} -> error\n  end.\n\npop_k(Tx, K) ->\n  {QS, QE} = erlfdb_tuple:range({?NAME, ?VAL}),\n  case erlfdb:get_range(Tx, QS, QE, [{limit, K}, {wait, true}]) of\n    [] ->\n      {{error, empty}, []};\n    KVs=[{S, _}|_] ->\n      N = length(KVs),\n      {E, _} = lists:last(KVs),\n      erlfdb:clear_range(Tx, S, erlfdb_key:strinc(E)),\n      erlfdb:add(Tx, erlfdb_tuple:pack({?NAME, ?NPOP}), N),\n\n      Status = if N == K -> ok; true -> {error, empty} end,\n      {Status, [ V || {_, V} <- KVs ]}\n  end.\n\nlen(Tx) ->\n  F = [\n    erlfdb:get(Tx, erlfdb_tuple:pack({?NAME, ?NPUSH})),\n    erlfdb:get(Tx, erlfdb_tuple:pack({?NAME, ?NPOP}))\n  ],\n  [Npush, Npop] = [ decode_as_int(X, 0) || X <- erlfdb:wait_for_all(F)],\n  Npush - Npop.\n\nqueue(Tx) ->\n  {QS, QE} = erlfdb_tuple:range({?NAME, ?VAL}),\n  KVs = erlfdb:get_range(Tx, QS, QE, [{wait, true}]),\n  [ V || {_, V} <- KVs ].\n\ndecode_as_int(not_found, Default) -> Default;\ndecode_as_int(Val, _Default) -> binary:decode_unsigned(Val, little).\n\nconsume_k(Tx, K) ->\n  case pop_k(Tx, K) of\n    {ok, Vals} ->\n      {Vals, undefined};\n    {{error, empty}, Vals} ->\n      {Vals, watch(Tx)}\n  end.\n\nwatch(Tx) ->\n  erlfdb:watch(Tx, erlfdb_tuple:pack({?NAME, ?NPUSH})).\n\n```\n\nFunctions `push/2` and `pop_k/2` are the two most important functions to focus on for now.\n\nWe'll discuss `consume_k/2`, and `watch/1` later, when we define our consumer.","ref":"kv_queue.html#code-time"},{"type":"extras","title":"Queue push - KvQueue - A distributed durable queue with erlfdb","doc":"When we push an item onto the queue, we want to make sure that there is no key contention. So,\n\n1. We insert the item into the `val` keyspace with an incomplete versionstamp in the key. This ensures that `n` parallel transactions will get `n` unique versionstamps and insert into the queue in the order of the serialized commits, with no key conflicts.\n2. We do an atomic add on the `npush` key. FDB atomic operations are CRDTs. That is, FDB can resolve parallel atomic operations without key conflicts. An interesting consequence of this is that our client does not know the value of the counter when incrementing it.","ref":"kv_queue.html#queue-push"},{"type":"extras","title":"Queue pop - KvQueue - A distributed durable queue with erlfdb","doc":"When we pop `k` items from the front of the queue, we want to make sure we safely delete them in the same transaction.\n\n1. A `get_range` with `limit` will read up to `k` items in the range and then stop.\n2. When values are found, we use `clear_range` carefully to only touch those we've retrieved, and increment the `npop` counter accordingly.\n3. Smart return values will help our consumer know when to keep consuming and when to wait.","ref":"kv_queue.html#queue-pop"},{"type":"extras","title":"Try it out! - KvQueue - A distributed durable queue with erlfdb","doc":"We can push any item represented by a `binary()` onto the queue. If you execute this multiple times, a `<<\"hello\">>` item will be added each time.\n\n```erlang\nerlfdb:transactional(Db, fun(Tx) -> kv_queue:push(Tx, <<\"hello\">>) end).\n```\n\nWe can view the entire queue at once. This is not how we will be consuming items, but it's nice to see them all at the same time.\n\n```erlang\nerlfdb:transactional(Db, fun kv_queue:queue/1).\n```\n\nWe can pop a single item from the front of the queue. If you execute this until the queue is empty, you'll receive `error` as the response. That just means that the queue is empty.\n\n```erlang\nerlfdb:transactional(Db, fun kv_queue:pop/1).\n```\n\nIf you've emptied the queue, the length will be `0`. Feel free to go back and add some more items.\n\n```erlang\nerlfdb:transactional(Db, fun kv_queue:len/1).\n```","ref":"kv_queue.html#try-it-out"},{"type":"extras","title":"Defining the Consumer - KvQueue - A distributed durable queue with erlfdb","doc":"Our consumer is a `gen_server`. We'll start any number of them, and each one will monitor the queue for items. There is no coordination between consumers. They will each try to empty the queue as quickly as possible, and then wait for a signal to start once again.\n\nThe consumer will be configurable to consume `K` items from a single transaction. Increasing `K` increases consumer throughput quite dramatically.\n\nNote: consumers will have lots of key conflicts with each other because they are all vying for the items at the front of the queue. Other clever optimizations can be done to reduce conflicts, but we're skipping that for now.\n\n```erlang\n-module(kv_consumer).\n-export([\n         start_link/1,\n         init/1,\n         handle_cast/2,\n         handle_call/3,\n         handle_info/2\n]).\n\n-behaviour(gen_server).\n\nstart_link(InitArgs) ->\n  gen_server:start_link(?MODULE, InitArgs, []).\n\ninit([Db, K]) ->\n  gen_server:cast(self(), consume),\n  {ok, #{db => Db, k => K, watch => undefined}}.\n\nhandle_cast(consume, S) ->\n  #{db := Db, k := K} = S,\n  {Items, Watch} =\n          erlfdb:transactional(Db,\n                  fun(Tx) -> kv_queue:consume_k(Tx, K) end\n          ),\n  [ log(\"Consumed items ~p~n\", [Items]) || Items =/= [] ],\n  case Watch of\n    undefined ->\n      gen_server:cast(self(), consume);\n    _ ->\n      log(\"Waiting for push~n\", [])\n  end,\n  {noreply, S#{watch => Watch}}.\n\nhandle_call(_Call, _From, _S) ->\n  erlang:error(function_clause).\n\nhandle_info({Ref, ready}, S=#{watch := {erlfdb_future, Ref, _}}) ->\n  gen_server:cast(self(), consume),\n  {noreply, S#{watch => undefined}};\nhandle_info(_Info, S) ->\n  {noreply, S}.\n\nlog(F, A) ->\n  io:format(\"~p: \" ++ F, [self()|A]).\n```","ref":"kv_queue.html#defining-the-consumer"},{"type":"extras","title":"Watch for pushes - KvQueue - A distributed durable queue with erlfdb","doc":"Up above we skipped over the `watch/2` function. Here it shines. Via `consume_k/2`, when the queue is empty, the consumer receives a \"watch\" on the `npush` counter. Whenever the `npush` key's value is changed, FDB resolves all watches on that key: each process that created a watch receives a `{reference(), ready}` message on its message queue.\n\nRemember earlier we decided to track the queue length with two counters instead of one? This is why: we don't want our consumers to be signaled by other consumers; we only want to know when new items are available. By splitting the queue length into `npush` and `npop`, we allow watch creation only on `npush`.\n\nThe consumer uses the watch to enter an idle state while the queue is empty, so we don't have to continuously poll for the existence of queue items.\n\nIn summary, `consume_k/2` will attempt to consume `K` items from the front of the queue. If any number less than `K` is found, then a watch is created, and the consumer goes into an idle state.","ref":"kv_queue.html#watch-for-pushes"},{"type":"extras","title":"Processing queue items - KvQueue - A distributed durable queue with erlfdb","doc":"It's critically important to do all processing of queue items **outside of the FDB transaction**. For our simple case, we're just logging the items, but notice that our log is not within the `erlfdb:transactional/2` function. Because a transaction function can execute multiple times for a single commit (e.g. when there are key conflicts), side-effects inside transactions are quite dangerous.","ref":"kv_queue.html#processing-queue-items"},{"type":"extras","title":"Supervisor - KvQueue - A distributed durable queue with erlfdb","doc":"This is a standard `simple_one_for_one` supervisor. Nothing special about it. We're defining it here to help us keep track of all the consumers we're about to start.\n\n```erlang\n-module(kv_consumer_sup).\n-behaviour(supervisor).\n\n-export([start_link/0, start_child/2, terminate_all_children/0, init/1]).\n\nstart_link() -> supervisor:start_link({local, ?MODULE}, ?MODULE, []).\n\nstart_child(Db, K) ->\n  supervisor:start_child(?MODULE, [[Db, K]]).\n\nterminate_all_children() ->\n  [ supervisor:terminate_child(?MODULE, X)\n   || {_, X, _, _} <- supervisor:which_children(?MODULE) ].\n\ninit([]) ->\n  SupFlags = #{strategy => simple_one_for_one},\n  ChildSpec = #{\n                id => kv_consumer,\n                start => {kv_consumer, start_link, []}\n  },\n  {ok, {SupFlags, [ChildSpec]}}.\n\n```\n\nLet's get our supervisor running.\n\n```erlang\nkv_consumer_sup:start_link().\n```\n\nNow, we can start any number of consumers we want. You can even create them with different `K` values if you'd like. If your queue has any items in it, the consumer will process them immediately once it's started.\n\n```erlang\nkv_consumer_sup:start_child(Db, 1).\n```","ref":"kv_queue.html#supervisor"},{"type":"extras","title":"Defining the Producer - KvQueue - A distributed durable queue with erlfdb","doc":"Finally, we need a bit of code that will throw items onto the queue with reckless abandon. Good thing we have BEAM.\n\nWhen you execute this, you should expect the producer to finish before the consumers, assuming you've kept the consumers' `K` value in the single digits. Either way, the producer should be very quick because there are no key conflicts.\n\nYou'll see the log output from each consumer in the meantime, and a \"Waiting for push\" log when each goes idle.\n\n```erlang\nPushFun = fun(X) ->\n  erlfdb:transactional(Db, fun(Tx) -> kv_queue:push(Tx, integer_to_binary(X)) end)\nend,\n\nSeq = lists:seq(1, 1000),\n\ntimer:tc(fun() ->\n           Self = self(),\n           [ spawn(fun() -> PushFun(X), Self ! ok end) || X <- Seq ],\n           (fun\n             Recv(0) -> ok;\n             Recv(N) -> receive ok -> Recv(N-1) end\n            end)(length(Seq))\n        end).\n```","ref":"kv_queue.html#defining-the-producer"},{"type":"extras","title":"Cleanup (optional) - KvQueue - A distributed durable queue with erlfdb","doc":"To cleanup, you can stop the supervisor and delete the queue.\n\n```erlang\ncatch(gen_server:stop(whereis(kv_consumer_sup))),\nerlfdb:transactional(Db, fun kv_queue:delete/1).\n```","ref":"kv_queue.html#cleanup-optional"}],"content_type":"text/plain","producer":{"name":"ex_doc","version":[48,46,51,50,46,50]}}